from policyengine_us.model_api import *


class surviving_spouse_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    label = "Qualifies for surviving spouse filing status"
    reference = "https://www.law.cornell.edu/uscode/text/26/2#a"

    def formula(tax_unit, period, parameters):
        # The surviving spouse filing status should only apply to surviving spouse heads
        # who maintain a household for at least one dependent
        person = tax_unit.members
        is_head = person("is_tax_unit_head", period)
        is_surviving_spouse = person("is_surviving_spouse", period)
        surviving_spouse_head = tax_unit.any(is_head & is_surviving_spouse)
        has_child_dependents = (
            tax_unit("tax_unit_child_dependents", period) > 0
        )
        married = tax_unit("tax_unit_married", period)
        return surviving_spouse_head & has_child_dependents & ~married



from policyengine_us.model_api import *


class ct_amt(Variable):
    value_type = float
    entity = TaxUnit
    label = "Connecticut alternative minimum tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.CT
    reference = (
        "https://portal.ct.gov/-/media/DRS/Forms/2022/Income/2022-CT-1040-Instructions_1222.pdf#page=2"  # line 9
        "https://portal.ct.gov/-/media/DRS/Forms/2022/Income/CT-1040_1222.pdf#page=1"  # line 9
        "https://portal.ct.gov/-/media/DRS/Forms/2022/Income/CT-6251_1222.pdf#page=1"  # line 23
        "https://www.irs.gov/pub/irs-pdf/f6251.pdf#page=1"  # line 7
        "https://www.irs.gov/pub/irs-pdf/i6251.pdf#page=9"  # line 7
    )



from policyengine_us.model_api import *


class mt_interest_exemption_eligible_person(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for the Montana interest exemption"
    definition_period = YEAR
    reference = "https://mtrevenue.gov/wp-content/uploads/dlm_uploads/2022/12/Form-2-2022-Instructions.pdf#page=25"
    defined_for = StateCode.MT

    def formula(person, period, parameters):
        p = parameters(period).gov.states.mt.tax.income.exemptions

        if p.applies:
            head_or_spouse = person("is_tax_unit_head_or_spouse", period)
            age = person("age", period)
            return (
                person.tax_unit.any(age >= p.interest.age_threshold)
                & head_or_spouse
            )

        return False



from policyengine_us.model_api import *


class nd_income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "North Dakota income tax before refundable credits"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2021-iit/form-nd-1-2021.pdf"
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2021-iit/individual-income-tax-booklet-2021.pdf"
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2022-iit/form-nd-1-2022.pdf"
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2022-iit/2022-individual-income-tax-booklet.pdf"
    )
    defined_for = StateCode.ND

    def formula(tax_unit, period, parameters):
        itax_before_credits = tax_unit("nd_income_tax_before_credits", period)
        nonrefundable_credits = tax_unit("nd_nonrefundable_credits", period)
        return max_(0, itax_before_credits - nonrefundable_credits)



from policyengine_us.model_api import *


class co_ccap_subsidy(Variable):
    value_type = float
    entity = SPMUnit
    label = "Colorado Child Care Assistance Program"
    unit = USD
    definition_period = MONTH
    defined_for = "co_ccap_eligible"

    def formula(spm_unit, period, parameters):
        expenses = add(
            spm_unit,
            period,
            ["pre_subsidy_childcare_expenses", "after_school_expenses"],
        )
        parent_fee = spm_unit("co_ccap_parent_fee", period)
        return max_(expenses - parent_fee, 0)



from policyengine_us.model_api import *


class tuition_and_fees_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Tuition and fees deduction"
    unit = USD
    defined_for = "tuition_and_fees_deduction_eligible"
    definition_period = YEAR
    reference = (
        "https://www.irs.gov/pub/irs-pdf/f8917.pdf#page=2"
        # Law was repealed in 2021
        "https://irc.bloombergtax.com/public/uscode/doc/irc/section_222"
    )

    def formula(tax_unit, period, parameters):
        qualified_tuition_expenses = add(
            tax_unit, period, ["qualified_tuition_expenses"]
        )
        adjusted_gross_income = tax_unit("adjusted_gross_income", period)
        p = parameters(period).gov.irs.deductions.tuition_and_fees
        filing_status = tax_unit("filing_status", period)
        joint = filing_status == filing_status.possible_values.JOINT
        cap = where(
            joint,
            p.joint.calc(adjusted_gross_income),
            p.non_joint.calc(adjusted_gross_income),
        )
        return min_(qualified_tuition_expenses, cap)



from policyengine_us.model_api import *


class pr_earned_income_credit_eligible_person(Variable):
    value_type = bool
    entity = Person
    label = "Puerto Rico earned income credit eligible person"
    definition_period = YEAR
    reference = "https://hacienda.pr.gov/sites/default/files/schedule_ct_rev._jul_5_23_informative_-_instructions.pdf#page=1"

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.territories.pr.tax.income.credits.earned_income
        head_or_spouse = person("is_tax_unit_head_or_spouse", period)

        unearned_income = person(
            "pr_earned_income_credit_unearned_income", period
        )
        unearned_income_amount_under_limit = (
            unearned_income <= p.unearned_income.limit
        )

        age = person("age", period)
        age_within_range = age >= p.eligibility.min

        filing_status = person.tax_unit("filing_status", period)
        not_separate = filing_status != filing_status.possible_values.SEPARATE

        eligible = (
            head_or_spouse
            & unearned_income_amount_under_limit
            & age_within_range
        )
        # if separate filers are eligible
        if p.eligibility.separate_filer:
            return eligible
        # else, filing status can't be filing separate
        else:
            return eligible & not_separate



from policyengine_us.model_api import *


class ca_yctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "California Young Child Tax Credit"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?lawCode=RTC&sectionNum=17052.1",
        "https://www.ftb.ca.gov/forms/2021/2021-3514-instructions.html",
        "https://www.ftb.ca.gov/forms/2021/2021-3514.pdf#page=3",
        "https://www.ftb.ca.gov/forms/2022/2022-3514-instructions.html",
        "https://www.ftb.ca.gov/forms/2022/2022-3514.pdf#page=3",
    )
    defined_for = StateCode.CA

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        p = parameters(period).gov.states.ca.tax.income.credits.young_child

        # determine eligibility, which requires both (a) and (b) to be true:
        # (a) tax unit has at least one CalEITC-qualifying child
        # (b) tax unit receives CalEITC _OR_ is CalEITC eligible but has losses
        # ... determine (a)
        meets_age_limit = person("age", period) < p.ineligible_age
        is_qualifying_child_for_caleitc = person(
            "ca_is_qualifying_child_for_caleitc", period
        )
        is_eligible_child = meets_age_limit & is_qualifying_child_for_caleitc
        has_eligible_child = tax_unit.any(is_eligible_child)
        # ... determine (b) part one
        gets_caleitc = tax_unit("ca_eitc", period) > 0
        # ... determine (b) part two
        is_caleitc_eligible = tax_unit("ca_eitc_eligible", period)
        # ... ... determine if losses are limited to modest amount
        federal_gross_income = add(tax_unit, period, ["irs_gross_income"])
        total_federal_net_loss = max_(0, -federal_gross_income)
        has_limited_losses = total_federal_net_loss <= p.loss_threshold
        # ... ... determine if earnings are limited to modest amount
        total_earnings = tax_unit("tax_unit_earned_income", period)
        has_limited_earnings = total_earnings <= p.loss_threshold
        # ... ... combine all the (b) elements where appropriate
        is_loss_eligible = where(
            p.loss_threshold > 0,
            is_caleitc_eligible & has_limited_losses & has_limited_earnings,
            False,
        )
        # ... combine (a) and (b) parts to determine eligibility
        eligible = has_eligible_child & (gets_caleitc | is_loss_eligible)

        # phase out credit amount
        eitc_earnings = tax_unit("filer_adjusted_earnings", period)
        excess_earnings = max_(0, eitc_earnings - p.phase_out.start)
        increments = excess_earnings / p.phase_out.increment
        reduction = increments * p.phase_out.amount

        return eligible * max_(0, p.amount - reduction)



from policyengine_us.model_api import *


class nj_senior_exemption(Variable):
    value_type = float
    entity = TaxUnit
    label = "New Jersey senior exemption"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.NJ

    def formula(tax_unit, period, parameters):
        # First get their filing status.
        filing_status = tax_unit("filing_status", period)

        # Then get the NJ senior exemptions part of the parameter tree
        p = parameters(period).gov.states.nj.tax.income.exemptions.senior

        # Get the individual filer's age.
        age_head = tax_unit("age_head", period)

        # Determine if head of household (filer) is eligible.
        head_eligible = (age_head >= p.age_threshold).astype(int)

        # Get the spouse age, if applicable.
        age_spouse = tax_unit("age_spouse", period)

        # Determine whether spouse is eligible (>= age 65).
        joint = filing_status == filing_status.possible_values.JOINT
        spouse_eligible = ((age_spouse >= p.age_threshold) * joint).astype(int)

        # Calculate total senior exemption.
        return (head_eligible + spouse_eligible) * p.amount



from policyengine_us.model_api import *


class ne_school_readiness_credit_child_care_worker_rating(Variable):
    value_type = int
    entity = Person
    label = "Level of child care worker for the Nebraska school readiness refundable tax credit"
    definition_period = YEAR
    defined_for = StateCode.NE
    reference = "https://revenue.nebraska.gov/sites/default/files/doc/tax-forms/2024/f_Individual_Income_Tax_Booklet.pdf#page=2"



from policyengine_us.model_api import *


class student_loan_interest_ald(Variable):
    value_type = float
    entity = Person
    label = "Student loan interest ALD"
    unit = USD
    definition_period = YEAR
    documentation = "Above-the-line deduction for student loan interest"
    reference = "https://www.law.cornell.edu/uscode/text/26/221"
    defined_for = "student_loan_interest_ald_eligible"

    def formula(person, period, parameters):
        interest = person("student_loan_interest", period)
        p = parameters(period).gov.irs.ald.student_loan_interest
        filing_status = person.tax_unit("filing_status", period)
        cap = p.cap[filing_status]
        capped_interest = min_(interest, cap)
        joint = filing_status == filing_status.possible_values.JOINT
        # Combine the income for units filing jointly
        modified_agi = person("student_loan_interest_ald_magi", period)
        combined_magi = where(
            joint,
            person.tax_unit.sum(modified_agi),
            modified_agi,
        )
        reduction_start = p.reduction.start[filing_status]
        income_excess = max_(0, combined_magi - reduction_start)
        divisor = p.reduction.divisor[filing_status]
        reduction_rate = np.zeros_like(divisor)
        mask = divisor != 0
        reduction_rate[mask] = income_excess[mask] / divisor[mask]
        reduction_amount = capped_interest * reduction_rate
        head_or_spouse = person("is_tax_unit_head_or_spouse", period)
        return max_(capped_interest - reduction_amount, 0) * head_or_spouse



from policyengine_us.model_api import *


class mt_salt_deduction(Variable):
    value_type = float
    entity = Person
    label = "Montana state and local tax deduction"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://mtrevenue.gov/wp-content/uploads/dlm_uploads/2023/05/Montana-Idividiual-Income-Tax-Return-Form-2-2022v6.2.pdf#page=7"
        "https://law.justia.com/codes/montana/2022/title-15/chapter-30/part-21/section-15-30-2131/"
        # Montana Code Annotated MT Code § 15-30-2131 (2022) (1)(a)
    )
    defined_for = StateCode.MT

    def formula(person, period, parameters):
        # Not included the SALT tax, only real estate taxes
        p = parameters(period).gov.irs.deductions
        filing_status = person.tax_unit("filing_status", period)
        real_estate_tax = person("real_estate_taxes", period)
        return min_(
            real_estate_tax,
            p.itemized.salt_and_real_estate.cap[filing_status],
        )



from policyengine_us.model_api import *


class pa_tanf_age_eligible_on_pregnant_women_limitation(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Pennsylvania TANF age eligibility on pregnant women requirement"
    definition_period = YEAR
    defined_for = StateCode.PA

    def formula(spm_unit, period, parameters):
        person = spm_unit.members
        p = parameters(period).gov.states.pa.dhs.tanf.pregnancy_eligibility
        age = person("age", period)
        is_eligible_age = age < p.age_limit
        is_pregnant = person("is_pregnant", period)
        is_children_receiving_tanf = spm_unit("pa_tanf_age_eligible", period)
        # Must be pregnant and either of a qualifying age or children not receiving TANF.
        eligible = is_pregnant & (
            is_eligible_age | ~is_children_receiving_tanf
        )
        return spm_unit.any(eligible)



from policyengine_us.model_api import *


class pr_compensatory_low_income_credit(Variable):
    value_type = float
    unit = USD
    entity = Person
    label = "Additional compensatory low income credit"
    definition_period = YEAR
    reference = "https://casetext.com/statute/laws-of-puerto-rico/title-thirteen-taxation-and-finance/subtitle-17-internal-revenue-code-of-2011/part-ii-income-taxes/chapter-1007-credits-against-tax/subchapter-b-refundable-credits/30212-credit-for-low-income-individuals-older-than-sixty-five-65-years-of-age"
    defined_for = "pr_low_income_credit_eligible"

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.territories.pr.tax.income.credits.low_income.amount
        pension_income = person("pension_income", period)
        return p.additional.calc(pension_income, right=True)



from policyengine_us.model_api import *


def create_rent_relief_tax_credit() -> Reform:
    class rent_relief_tax_credit(Variable):
        value_type = float
        entity = TaxUnit
        label = "Rent Relief Tax Credit"
        unit = USD
        definition_period = YEAR
        reference = "https://www.congress.gov/bill/116th-congress/senate-bill/1106/text"

        def formula(tax_unit, period, parameters):
            # The applicable rent amount is capped at fair market rent
            rent = add(tax_unit, period, ["rent"])
            p = parameters(
                period
            ).gov.contrib.harris.rent_relief_act.rent_relief_credit
            safmr = tax_unit.household("small_area_fair_market_rent", period)

            gross_income = add(tax_unit, period, ["irs_gross_income"])
            # Reduce the income (equivalent to raising thresholds) for households
            # in high income areas, as defined by using SAFMR for HCV.

            rent_cap = safmr * p.safmr_share_rent_cap
            capped_rent = min_(rent, rent_cap)
            gross_income_fraction = (
                p.rent_income_share_threshold * gross_income
            )
            rent_excess = max_(capped_rent - gross_income_fraction, 0)
            safmr_used_for_hcv = tax_unit.household(
                "safmr_used_for_hcv", period
            )
            high_income_reduction = (
                safmr_used_for_hcv * p.high_income_area_threshold_increase
            )
            applicable_gross_income = max_(
                gross_income - high_income_reduction, 0
            )
            applicable_percentage = p.applicable_percentage.calc(
                applicable_gross_income, right=True
            )
            amount_if_rent_not_subsidized = applicable_percentage * rent_excess
            housing_assistance = tax_unit.spm_unit(
                "housing_assistance", period
            )
            rent_is_subsidized = housing_assistance > 0
            reduced_rent = max_(0, rent - housing_assistance)
            amount_if_rent_subsidized = reduced_rent * p.subsidized_rent_rate
            return where(
                rent_is_subsidized,
                amount_if_rent_subsidized,
                amount_if_rent_not_subsidized,
            )

    class income_tax_refundable_credits(Variable):
        value_type = float
        entity = TaxUnit
        definition_period = YEAR
        label = "federal refundable income tax credits"
        unit = USD

        def formula(tax_unit, period, parameters):
            p = parameters(period).gov.irs.credits
            previous_credits = add(tax_unit, period, p.refundable)
            rent_relief_credit = tax_unit("rent_relief_tax_credit", period)
            return rent_relief_credit + previous_credits

    class reform(Reform):
        def apply(self):
            self.update_variable(rent_relief_tax_credit)
            self.update_variable(income_tax_refundable_credits)

    return reform


def create_rent_relief_tax_credit_reform(
    parameters, period, bypass: bool = False
):
    if bypass:
        return create_rent_relief_tax_credit()

    p = parameters(
        period
    ).gov.contrib.harris.rent_relief_act.rent_relief_credit

    if p.in_effect:
        return create_rent_relief_tax_credit()
    else:
        return None


rent_relief_tax_credit = create_rent_relief_tax_credit_reform(
    None, None, bypass=True
)



from policyengine_us.model_api import *


class ca_additions(Variable):
    value_type = float
    entity = TaxUnit
    label = "CA AGI additions to federal AGI"
    unit = USD
    definition_period = YEAR
    reference = "https://www.ftb.ca.gov/forms/2021/2021-540.pdf"
    defined_for = StateCode.CA



from policyengine_us.model_api import *


class meets_ctc_child_identification_requirements(Variable):
    value_type = bool
    entity = Person
    definition_period = YEAR
    label = "Child meets CTC identification requirements"
    reference = "https://www.law.cornell.edu/uscode/text/26/24#h_7"

    def formula(person, period, parameters):
        p = parameters(period).gov.irs.credits.ctc
        if p.child_ssn_requirement_applies:
            ssn_card_type = person("ssn_card_type", period)
            ssn_card_str = ssn_card_type.decode_to_str()
            p = parameters(period).gov.irs.credits.ctc
            return np.isin(ssn_card_str, p.eligible_ssn_card_type)
        return True



from policyengine_us.model_api import *


class ct_section_179_expense_add_back(Variable):
    value_type = float
    entity = TaxUnit
    label = "Connecticut Section 179 Expense Add Back"
    definition_period = YEAR
    documentation = "Add 80 percent of the section 179 amount deducted in determining federal AGI."
    reference = "https://portal.ct.gov/-/media/DRS/Forms/2022/Income/2022-CT-1040-Instructions_1222.pdf#page=7"
    defined_for = StateCode.CT
    # use federal variables if they are added later



from policyengine_us.model_api import *


class ma_part_c_gross_income(Variable):
    value_type = float
    entity = TaxUnit
    label = "MA Part C gross income"
    unit = USD
    definition_period = YEAR
    reference = "https://www.mass.gov/info-details/mass-general-laws-c62-ss-2"  # (b)(3)
    defined_for = StateCode.MA

    def formula(tax_unit, period, parameters):
        return max_(0, add(tax_unit, period, ["long_term_capital_gains"]))



from policyengine_us.model_api import *


class StateGroup(Enum):
    CONTIGUOUS_US = "Contiguous US"
    AK = "Alaska"
    HI = "Hawaii"
    GU = "Guam"
    PR = "Puerto Rico"
    VI = "Virgin Islands"
    # Omit other territories for now.


class state_group(Variable):
    value_type = Enum
    possible_values = StateGroup
    default_value = StateGroup.CONTIGUOUS_US
    entity = Household
    label = "State group"
    definition_period = YEAR

    def formula(household, period, parameters):
        NON_CONTIGUOUS_STATES = ("AK", "HI", "GU", "PR", "VI")
        state_code = household("state_code", period).decode_to_str()
        return where(
            np.isin(state_code, NON_CONTIGUOUS_STATES),
            StateGroup.encode(state_code).decode(),
            StateGroup.CONTIGUOUS_US,
        )



from policyengine_us.model_api import *


class nc_scca_child_age_eligible(Variable):
    value_type = bool
    entity = Person
    label = "North Carolina child age eligibility for Subsidized Child Care Assistance (SCCA) program"
    reference = "https://ncchildcare.ncdhhs.gov/Portals/0/documents/pdf/A/ACF-118_CCDF_FFY_2022-2024_For_North_Carolina_Amendment_1.pdf?ver=C9YfIUPAFekeBA3I1mN8aA%3d%3d#page=83"
    definition_period = YEAR
    defined_for = StateCode.NC

    def formula(person, period, parameters):
        p = parameters(period).gov.states.nc.ncdhhs.scca.age.limit

        is_disabled = person("is_disabled", period)
        age = person("age", period)

        # Apply appropriate age limit based on disability status
        age_limit = where(is_disabled, p.disabled, p.non_disabled)

        return age < age_limit



from policyengine_us.model_api import *


class mn_itemized_deductions(Variable):
    value_type = float
    entity = TaxUnit
    label = "Minnesota itemized deductions"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.revenue.state.mn.us/sites/default/files/2021-12/m1_21_0.pdf"
        "https://www.revenue.state.mn.us/sites/default/files/2023-01/m1_inst_21.pdf"
        "https://www.revenue.state.mn.us/sites/default/files/2022-12/m1_22.pdf"
        "https://www.revenue.state.mn.us/sites/default/files/2023-03/m1_inst_22.pdf"
    )
    defined_for = StateCode.MN

    def formula(tax_unit, period, parameters):
        # 2021 Form M1 instructions say:
        #   You may claim the Minnesota standard deduction or itemize
        #   your deductions on your Minnesota return. You will generally
        #   pay less Minnesota income tax if you take the larger of your
        #   itemized or standard deduction.
        # ... calculate pre-limitation itemized deductions
        itm_deds_less_salt = tax_unit("itemized_deductions_less_salt", period)
        capped_property_taxes = tax_unit("capped_property_taxes", period)
        mn_itm_deds = itm_deds_less_salt + capped_property_taxes
        # ... calculate itemized deductions offset
        p = parameters(period).gov.states.mn.tax.income.deductions.itemized
        exempt_deds = add(
            tax_unit,
            period,
            ["medical_expense_deduction", "casualty_loss_deduction"],
        )
        net_deds = max_(0, mn_itm_deds - exempt_deds)
        filing_status = tax_unit("filing_status", period)
        lower_reduction_rate = p.reduction.excess_agi_fraction.low
        lower_reduction_threshold = p.reduction.agi_threshold.low[
            filing_status
        ]
        agi = tax_unit("adjusted_gross_income", period)
        lower_excess = max_(0, agi - lower_reduction_threshold)
        alternate_reduction_amount = p.reduction.alternate.rate * net_deds
        if p.alternate_reduction_applies:
            higher_reduction_threshold = p.reduction.agi_threshold.high[
                filing_status
            ]

            spread = higher_reduction_threshold - lower_reduction_threshold
            lower_reduction_amount = lower_reduction_rate * min_(
                lower_excess, spread
            )
            higher_reduction_rate = p.reduction.excess_agi_fraction.high
            higher_excess = max_(0, agi - higher_reduction_threshold)
            higher_reduction_amount = higher_reduction_rate * higher_excess
            main_reduction_amount = (
                lower_reduction_amount + higher_reduction_amount
            )
            alternate_reduction_applies = (
                agi > p.reduction.alternate.income_threshold
            )
            smaller_reduction_amount = min_(
                alternate_reduction_amount, main_reduction_amount
            )
            reduction = where(
                alternate_reduction_applies,
                alternate_reduction_amount,
                smaller_reduction_amount,
            )
        else:
            # ... calculate pre-limitation amount
            excess_agi = max_(0, agi - lower_reduction_threshold)
            main_reduction_amount = lower_reduction_rate * excess_agi
            reduction = min_(alternate_reduction_amount, main_reduction_amount)
        return max_(0, mn_itm_deds - reduction)



from policyengine_us.model_api import *


class ca_state_supplement(Variable):
    value_type = float
    entity = SPMUnit
    label = "California state supplement"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.CA
    reference = "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?lawCode=WIC&sectionNum=12200"

    def formula(spm_unit, period, parameters):
        payment_standard = spm_unit(
            "ca_state_supplement_payment_standard", period
        )
        ssi = add(spm_unit, period, ["ssi"])
        countable_income = add(spm_unit, period, ["ssi_countable_income"])
        return max_(0, payment_standard - ssi - countable_income)



from policyengine_us.model_api import *


class ca_foster_care_minor_dependent(Variable):
    value_type = bool
    entity = Person
    label = "California foster care minor dependent"
    definition_period = MONTH
    defined_for = StateCode.CA

    def formula(person, period, parameters):
        age = person("monthly_age", period)
        p = parameters(period).gov.states.ca.foster_care
        age_eligible = age < p.age_threshold
        in_foster_care = person("is_in_foster_care", period)
        return age_eligible & in_foster_care



from policyengine_us.model_api import *


class ca_capi_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "California CAPI income eligible"
    definition_period = YEAR
    defined_for = StateCode.CA
    reference = "https://www.cdss.ca.gov/Portals/9/CAPI/CAPI_Regulations-Accessible.pdf"

    def formula(spm_unit, period, parameters):
        payment_standard = add(
            spm_unit,
            period,
            ["ssi_amount_if_eligible", "ca_state_supplement_payment_standard"],
        )
        person = spm_unit.members
        eligible_person = person("ca_capi_eligible_person", period)
        countable_income = spm_unit.sum(
            (person("ssi_countable_income", period) * eligible_person)
        )
        return payment_standard > countable_income



from policyengine_us.model_api import *


class ny_agi(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY adjusted gross income"
    unit = USD
    definition_period = YEAR
    reference = "https://casetext.com/regulation/new-york-codes-rules-and-regulations/title-20-department-of-taxation-and-finance/chapter-ii-income-taxes-and-estate-taxes/subchapter-a-new-york-state-personal-income-tax-under-article-22-of-the-tax-law/article-3-nonresidents/part-132-new-york-adjusted-gross-income-of-a-nonresident-individual/new-york-adjusted-gross-income-defined/section-1321-new-york-adjusted-gross-income-of-a-nonresident-individual"
    defined_for = StateCode.NY

    adds = ["adjusted_gross_income", "ny_additions"]
    subtracts = ["ny_agi_subtractions"]



from policyengine_us.model_api import *


class az_dependent_tax_credit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Arizona dependent tax credit"
    unit = USD
    documentation = "https://www.azleg.gov/viewdocument/?docName=https://www.azleg.gov/ars/43/01073-01.htm"
    definition_period = YEAR
    defined_for = StateCode.AZ

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        p = parameters(
            period
        ).gov.states.az.tax.income.credits.dependent_credit
        dependent = person("is_tax_unit_dependent", period)
        age = person("age", period)
        dependent_amount = p.amount.calc(age) * dependent
        amount = tax_unit.sum(dependent_amount)
        income = tax_unit("adjusted_gross_income", period)
        filing_status = tax_unit("filing_status", period)
        reduction_start = p.reduction.start[filing_status]
        excess = max_(income - reduction_start, 0)
        increments = np.ceil(excess / p.reduction.increment)
        reduction_percentage = min_(increments * p.reduction.percentage, 1)
        return amount * (1 - reduction_percentage)



from policyengine_us.model_api import *


class aged_blind_count(Variable):
    value_type = int
    entity = TaxUnit
    label = "Aged and or blind head and spouse count"
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/63#f"

    def formula(tax_unit, period, parameters):
        blind_head = tax_unit("blind_head", period).astype(int)
        blind_spouse = tax_unit("blind_spouse", period).astype(int)
        aged_head = tax_unit("aged_head", period).astype(int)
        aged_spouse = tax_unit("aged_spouse", period).astype(int)
        return blind_head + blind_spouse + aged_head + aged_spouse



from policyengine_us.model_api import *


class co_low_income_cdcc_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Eligible for the Colorado Low-income Child Care Expenses Credit"
    documentation = (
        "https://casetext.com/statute/colorado-revised-statutes/title-39-taxation/specific-taxes/income-tax/article-22-income-tax/part-1-general/section-39-22-1195-child-care-expenses-tax-credit-legislative-declaration-definitions"
        "https://tax.colorado.gov/sites/tax/files/documents/DR_104_Book_2022.pdf#page=46"
    )
    definition_period = YEAR
    defined_for = StateCode.CO

    def formula(tax_unit, period, parameters):
        no_fed_cdcc = tax_unit("capped_cdcc", period) <= 0
        p = parameters(period).gov.states.co.tax.income.credits
        fed_agi = tax_unit("adjusted_gross_income", period)
        agi_eligible = fed_agi <= p.cdcc.low_income.federal_agi_threshold
        return no_fed_cdcc & agi_eligible



from policyengine_us.model_api import *


class md_hundred_year_subtraction_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for the Maryland hundred year subtraction"
    definition_period = YEAR
    reference = "https://trackbill.com/bill/maryland-house-bill-186-income-tax-subtraction-modification-for-centenarians/2173534/"
    defined_for = StateCode.MD

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(
            period
        ).gov.states.md.tax.income.agi.subtractions.hundred_year
        return age >= p.age_threshold



from policyengine_us.model_api import *


class is_executive_administrative_professional(Variable):
    value_type = bool
    entity = Person
    label = "is employed in a bona fide executive, administrative, or professional capacity"
    reference = "https://www.law.cornell.edu/uscode/text/29/213 ; https://www.congress.gov/crs-product/IF12480"
    definition_period = YEAR



from policyengine_us.model_api import *


class ca_itemized_deductions(Variable):
    value_type = float
    entity = TaxUnit
    label = "California itemized deductions"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.ftb.ca.gov/forms/2021/2021-540-ca-instructions.html"
        "https://www.ftb.ca.gov/forms/2022/2022-540-ca-instructions.html"
    )
    defined_for = StateCode.CA

    def formula(tax_unit, period, parameters):
        itm_deds_max = tax_unit(
            "ca_itemized_deductions_pre_limitation", period
        )
        # compute high-AGI limit on itemized deductions
        p = parameters(period).gov.states.ca.tax.income.deductions.itemized
        # ... determine part of itemized deductions subject to limit
        excluded_itm_deds = add(tax_unit, period, p.limit.excluded_deductions)
        included_itm_deds = p.limit.ded_fraction * max_(
            0, itm_deds_max - excluded_itm_deds
        )
        # ... determine limit amount
        agi = tax_unit("adjusted_gross_income", period)
        filing_status = tax_unit("filing_status", period)
        agi_limit = p.limit.agi_fraction * max_(
            0, agi - p.limit.agi_threshold[filing_status]
        )
        limit_amount = min_(included_itm_deds, agi_limit)
        # return limited itemized deductions
        return max_(0, itm_deds_max - limit_amount)



from policyengine_us.model_api import *


class oh_senior_citizen_credit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Ohio senior citizen credit"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://tax.ohio.gov/static/forms/ohio_individual/individual/2021/pit-it1040-booklet.pdf#page=20",
        "https://codes.ohio.gov/ohio-revised-code/section-5747.055",
    )
    defined_for = StateCode.OH

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.oh.tax.income.credits.senior_citizen
        person = tax_unit.members
        head = person("is_tax_unit_head", period)
        head_has_not_taken_lump_sum_distribution = (
            ~person("oh_has_taken_oh_lump_sum_credits", period) * head
        )
        age_head = tax_unit("age_head", period)
        elderly_head = age_head >= p.age_threshold
        eligible_head = (
            tax_unit.any(head_has_not_taken_lump_sum_distribution)
            & elderly_head
        )
        agi = tax_unit("oh_modified_agi", period)
        exemptions = tax_unit("oh_personal_exemptions", period)
        applicable_income = max_(agi - exemptions, 0)
        credit_amount = p.amount.calc(applicable_income)
        return eligible_head * credit_amount



from policyengine_us.model_api import *


class taxsim_dep18(Variable):
    value_type = float
    entity = TaxUnit
    label = "Children under 13"
    unit = USD
    definition_period = YEAR

    adds = ["is_child_dependent"]



from policyengine_us.model_api import *


class vt_eitc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Vermont earned income tax credit"
    unit = USD
    definition_period = YEAR
    reference = "https://tax.vermont.gov/sites/tax/files/documents/IN-112-2022.pdf#page=1"
    defined_for = StateCode.VT

    def formula(tax_unit, period, parameters):
        federal_eitc = tax_unit("eitc", period)
        p = parameters(period).gov.states.vt.tax.income.credits.eitc
        child_dependents = tax_unit("tax_unit_child_dependents", period)

        rate = where(
            p.enhanced_structure.in_effect,
            p.enhanced_structure.rate.calc(child_dependents),
            p.match,
        )

        return federal_eitc * rate



from policyengine_us.model_api import *


class tax_exempt_interest_income(Variable):
    value_type = float
    entity = Person
    label = "tax-exempt interest income"
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/subtitle-A/chapter-1/subchapter-B/part-III"
    uprating = "calibration.gov.irs.soi.tax_exempt_interest_income"



from policyengine_us.model_api import *


class cohabitating_spouses(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    label = "Cohabitating spouses"
    documentation = (
        "Whether spouses in joint or separate tax units are cohabitating."
    )



from policyengine_us.model_api import *


class mt_deductions_indiv(Variable):
    value_type = float
    entity = Person
    label = "The total amount of Montana deductions and exemptions when married filing separately"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MT

    def formula(person, period, parameters):
        standard_deduction = person("mt_standard_deduction_indiv", period)
        itemized_deductions = person("mt_itemized_deductions_indiv", period)
        itemizes = person.tax_unit("mt_tax_unit_itemizes", period)
        return where(itemizes, itemized_deductions, standard_deduction)



from policyengine_us.model_api import *


class co_denver_property_tax_relief_renter_eligible(Variable):
    value_type = float
    entity = SPMUnit
    label = "Eligible for the renter Denver Property Tax Relief"
    defined_for = "in_denver"
    definition_period = YEAR
    reference = "https://library.municode.com/co/denver/codes/code_of_ordinances?nodeId=TITIIREMUCO_CH53TAMIRE_ARTXIREPRTAASELLCOPROWTE_S53-492DE"  # 53-495 (e)

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.local.co.denver.dhs
        pays_rent = add(spm_unit, period, ["rent"]) > 0
        has_elderly_or_disabled = spm_unit(
            "has_co_denver_dhs_elderly_disabled", period
        )
        renters_with_elderly_or_disabled = has_elderly_or_disabled & pays_rent

        income = spm_unit("co_denver_property_tax_relief_income", period)
        ami = spm_unit.household("ami", period)
        moderate_factor = spm_unit("hud_moderate_income_factor", period)
        size = spm_unit("spm_unit_size", period)
        ami_rate = p.property_tax_relief.ami_rate.renter.calc(size)
        renter_income_limit = ami * moderate_factor * ami_rate

        renter_income_eligible = income <= renter_income_limit

        return renters_with_elderly_or_disabled & renter_income_eligible



from policyengine_us.model_api import *


class il_cta_student_reduced_fare_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for the Illinois Chicago Transit Authority student reduced fare"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = (
        "https://www.transitchicago.com/reduced-fare-programs/#students"
    )

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.states.il.rta.cta.reduced_fare_program.age_threshold
        age = person("age", period)
        age_eligible = age <= p.student
        is_student = person("is_in_secondary_school", period)
        return age_eligible & is_student



from policyengine_us.model_api import *


class hud_annual_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "HUD annual income"
    unit = USD
    documentation = "Annual income for HUD programs"
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/cfr/text/24/5.609"

    adds = ["market_income", "social_security", "ssi", "tanf"]



from policyengine_us.model_api import *


class excess_payroll_tax_withheld(Variable):
    value_type = float
    entity = TaxUnit
    label = "Excess payroll (FICA/RRTA) tax withheld"
    unit = USD
    definition_period = YEAR



from policyengine_us.model_api import *


class nj_withheld_income_tax(Variable):
    value_type = float
    entity = Person
    label = "New Jersey withheld income tax"
    defined_for = StateCode.NJ
    unit = USD
    definition_period = YEAR

    def formula(person, period, parameters):
        agi = person("adjusted_gross_income_person", period)
        p = parameters(period).gov.states.nj.tax.income
        # Since New Jersey does not have a standard deduction
        # we apply the regular personal exemption amount
        personal_exemptions = p.exemptions.regular.amount["SINGLE"]
        reduced_agi = max_(agi - personal_exemptions, 0)
        return p.main.single.calc(reduced_agi)



from policyengine_us.model_api import *


class long_term_capital_gains_on_collectibles(Variable):
    value_type = float
    entity = Person
    label = "Long-term capital gains on collectibles"
    unit = USD
    documentation = "Portion of capital_gains_28_percent_rate_gain associated with collectibles."
    definition_period = YEAR
    reference = dict(
        title="26 U.S. Code § 1(h)(4)",
        href="https://www.law.cornell.edu/uscode/text/26/1#h_4",
    )



from policyengine_us.model_api import *


class is_mother(Variable):
    value_type = bool
    entity = Person
    label = "Is a mother"
    definition_period = YEAR

    def formula(person, period, parameters):
        # In the absence of relationship identifiers, check one of two
        # conditions:
        # 1. The person is female and has some children in their own household
        #    (provided in the CPS).
        # 2. Breastfeeding (user-input).
        female = person("is_female", period)
        has_children = person("is_parent", period)
        breastfeeding = person("is_breastfeeding", period)
        return breastfeeding | (female & has_children)



from policyengine_us.model_api import *


class ca_agi_subtractions(Variable):
    value_type = float
    entity = TaxUnit
    label = "CA AGI subtractions from federal AGI"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.ftb.ca.gov/forms/2021/2021-540-ca-instructions.html"
        "https://www.ftb.ca.gov/forms/2022/2022-540-ca-instructions.html"
    )
    defined_for = StateCode.CA
    adds = "gov.states.ca.tax.income.agi.subtractions"



from policyengine_us.model_api import *


class pr_retirement_deduction_eligibility(Variable):
    value_type = float
    entity = Person
    label = "Puerto Rico retirement contribution deduction eligibility"
    unit = USD
    definition_period = YEAR
    reference = "https://law.justia.com/codes/puerto-rico/title-thirteen/subtitle-17/part-ii/chapter-1005/subchapter-c/30135/"  # (7)(D)
    defined_for = StateCode.PR

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.territories.pr.tax.income.taxable_income.deductions.retirement

        # additionally no deduction if income is only from pension and annuities
        age = person("age", period)
        return age < p.age_threshold



from policyengine_us.model_api import *


class slcsp(Variable):
    value_type = float
    entity = TaxUnit
    label = "Second-lowest ACA silver-plan cost"
    unit = USD
    definition_period = MONTH
    adds = ["slcsp_age_curve_amount_person", "slcsp_family_tier_amount"]



from policyengine_us.model_api import *


class la_general_relief_gross_income(Variable):
    value_type = float
    entity = Person
    unit = USD
    label = "Gross Income sources accounted for under the Los Angeles County General Relief"
    definition_period = YEAR
    defined_for = "in_la"
    reference = "https://drive.google.com/file/d/1Oc7UuRFxJj-eDwTeox92PtmRVGnG9RjW/view?usp=sharing"

    adds = "gov.local.ca.la.general_relief.income_sources"



from policyengine_us.model_api import *


class lifetime_learning_credit_credit_limit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Lifetime Learning Credit credit limit"
    unit = USD
    documentation = "Value of the non-refundable Lifetime Learning Credit"
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/25A#c"

    def formula(tax_unit, period, parameters):
        income_tax_before_credits = tax_unit(
            "income_tax_before_credits", period
        )
        p = parameters(
            period
        ).gov.irs.credits.education.lifetime_learning_credit
        preceding_credits = add(tax_unit, period, p.preceding_credits)
        return max_(income_tax_before_credits - preceding_credits, 0)



from policyengine_us.model_api import *


class capital_gains_excluded_from_taxable_income(Variable):
    value_type = float
    entity = TaxUnit
    label = "Capital gains excluded from taxable income"
    unit = USD
    documentation = "This is subtracted from taxable income before applying the ordinary tax rates. Capital gains tax is calculated separately."
    definition_period = YEAR
    reference = dict(
        title="26 U.S. Code § 1(h)(1)(A)",
        href="https://www.law.cornell.edu/uscode/text/26/1#h_1_A",
    )

    def formula(tax_unit, period, parameters):
        net_capital_gain = tax_unit("net_capital_gain", period)
        adjusted_net_capital_gain = tax_unit(
            "adjusted_net_capital_gain", period
        )
        taxable_income = tax_unit("taxable_income", period)
        filing_status = tax_unit("filing_status", period)
        cg = parameters(period).gov.irs.capital_gains.brackets
        income_taxed_below_first_rate = clip(
            taxable_income, 0, cg.thresholds["1"][filing_status]
        )
        reduced_taxable_income = max_(
            taxable_income - net_capital_gain,
            min_(
                income_taxed_below_first_rate,
                taxable_income - adjusted_net_capital_gain,
            ),
        )
        return taxable_income - reduced_taxable_income



from policyengine_us.model_api import *


class il_aabd_expense_exemption_person(Variable):
    value_type = float
    entity = Person
    label = "Illinois Aid to the Aged, Blind or Disabled (AABD) expense exemption per person"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.IL
    reference = (
        "https://www.law.cornell.edu/regulations/illinois/Ill-Admin-Code-tit-89-SS-113.125",
    )

    adds = "gov.states.il.dhs.aabd.income.exemption.sources"



from policyengine_us.model_api import *


class hi_medical_expense_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Hawaii medical expense deduction"
    unit = USD
    documentation = (
        "https://files.hawaii.gov/tax/forms/2022/n11ins.pdf#page=15"
        "https://files.hawaii.gov/tax/forms/2022/n11ins.pdf#page=32"  # total itemized deduction worksheet
    )
    definition_period = YEAR
    defined_for = StateCode.HI

    def formula(tax_unit, period, parameters):
        p_deductions = parameters(period).gov.irs.deductions

        # 1. medical_expense_deduction: worksheet A-1
        # use hi_agi instead of AGI
        medical_expense = add(
            tax_unit, period, ["medical_out_of_pocket_expenses"]
        )
        hi_agi = tax_unit("hi_agi", period)
        medical_capped_amount = max_(
            0, p_deductions.itemized.medical.floor * hi_agi
        )
        return max_(0, medical_expense - medical_capped_amount)



from policyengine_us.model_api import *


class ma_liheap_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for the Massachusetts LIHEAP"
    definition_period = YEAR
    defined_for = "ma_liheap_income_eligible"
    reference = (
        "https://www.mass.gov/info-details/learn-about-home-energy-assistance-heap",
        "https://liheapch.acf.gov/tables/subsidize.htm#MA",
    )

    def formula(spm_unit, period, parameters):
        eligible_subsidized_housing = spm_unit(
            "ma_liheap_eligible_subsidized_housing", period
        )
        is_subsidized = spm_unit("receives_housing_assistance", period)
        return eligible_subsidized_housing | ~is_subsidized



from policyengine_us.model_api import *


class ny_college_tuition_credit_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "New York college tuition credit eligible"
    definition_period = YEAR
    reference = "https://www.nysenate.gov/legislation/laws/TAX/606"  # (t)
    defined_for = StateCode.NY

    def formula(tax_unit, period, parameters):
        return ~tax_unit("ny_itemizes", period)



from policyengine_us.model_api import *


class ks_disabled_veteran_exemptions_eligible_person(Variable):
    value_type = bool
    entity = Person
    label = "Eligible person for the Kansas disabled veteran exemption"
    unit = USD
    definition_period = YEAR
    reference = "https://law.justia.com/codes/kansas/chapter-79/article-32/section-79-32-121/"
    defined_for = StateCode.KS

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.states.ks.tax.income.exemptions.disabled_veteran
        disabled = person("is_permanently_and_totally_disabled", period)
        is_veteran = person("is_veteran", period)

        return p.in_effect * disabled * is_veteran



from policyengine_us.model_api import *


class me_agi_subtractions(Variable):
    value_type = float
    entity = TaxUnit
    label = "ME AGI subtractions"
    unit = USD
    documentation = "Subtractions from ME AGI over federal AGI."
    definition_period = YEAR
    defined_for = StateCode.ME
    reference = "https://www.maine.gov/revenue/sites/maine.gov.revenue/files/inline-files/22_1040me_sched_1s_ff.pdf"

    adds = [
        "tax_unit_taxable_social_security",
        "us_govt_interest",
        "me_pension_income_deduction",
    ]



from policyengine_us.model_api import *


class dc_eitc_with_qualifying_child(Variable):
    value_type = float
    entity = TaxUnit
    label = "DC EITC with qualifying children"
    unit = USD
    definition_period = YEAR
    reference = "https://code.dccouncil.gov/us/dc/council/code/sections/47-1806.04"  # (f)
    defined_for = StateCode.DC

    def formula(tax_unit, period, parameters):
        federal_eitc = tax_unit("eitc", period)
        p = parameters(period).gov.states.dc.tax.income.credits
        return federal_eitc * p.eitc.with_children.match



from policyengine_us.model_api import *


class pell_grant_dependent_contribution(Variable):
    value_type = float
    entity = Person
    unit = USD
    label = "Pell Grant dependent contribution"
    definition_period = YEAR

    def formula(person, period, parameters):
        assets = person("pell_grant_countable_assets", period)
        simplified = person("pell_grant_simplified_formula_applies", period)
        income = person("pell_grant_dependent_available_income", period)
        allowances = person("pell_grant_dependent_allowances", period)
        p = parameters(period).gov.ed.pell_grant.dependent
        adjusted_income = (income - allowances) * p.income_assessment_rate
        adjusted_assets = assets * p.asset_assessment_rate * ~simplified
        return adjusted_income + adjusted_assets



from policyengine_us.model_api import *


class pell_grant_max_fpg_percent_limit(Variable):
    value_type = float
    entity = Person
    unit = "/1"
    definition_period = YEAR
    label = "The maximum FPG percent to qualify for the maximum Pell Grant"

    def formula(person, period, parameters):
        household_type = person("pell_grant_household_type", period)
        limits = parameters(
            period
        ).gov.ed.pell_grant.sai.fpg_fraction.max_pell_limits

        return limits[household_type]



from policyengine_us.model_api import *


class taxable_self_employment_income(Variable):
    value_type = float
    entity = Person
    label = "Taxable self-employment income"
    definition_period = YEAR
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/1402#a"

    def formula(person, period, parameters):
        SEI_SOURCES = [
            "self_employment_income",
            "farm_income",
            "s_corp_self_employment_income",
        ]
        gross_sei = add(person, period, SEI_SOURCES)
        p = parameters(period).gov.irs
        combined_rate = (
            p.self_employment.rate.social_security
            + p.self_employment.rate.medicare
        )
        deduction_rate = p.ald.misc.employer_share * combined_rate
        net_sei = gross_sei * (1 - deduction_rate)
        # exclude net self-employment income below the reporting threshold.
        return where(
            net_sei < p.self_employment.net_earnings_exemption, 0, net_sei
        )



from policyengine_us.model_api import *


class ma_eaedc_dependent_care_deduction_person(Variable):
    value_type = float
    entity = Person
    unit = USD
    label = "Massachusetts EAEDC dependent care deduction for each person"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/massachusetts/106-CMR-704-275"  # (A)
    defined_for = StateCode.MA

    def formula(person, period, parameters):
        dependent = person("ma_eaedc_eligible_dependent", period)
        total_weekly_hours = person.spm_unit.sum(
            person("weekly_hours_worked", period.this_year)
        )
        age = person("monthly_age", period)
        p = parameters(
            period
        ).gov.states.ma.dta.tcap.deductions.dependent_care_expenses
        young_child = age < p.young_child_age_threshold
        amount = where(
            young_child,
            p.amount.younger.calc(total_weekly_hours),
            p.amount.older.calc(total_weekly_hours),
        )
        total_amount = amount * dependent
        care_expenses = person("pre_subsidy_childcare_expenses", period)
        return min_(total_amount, care_expenses)



from policyengine_us.model_api import *


class ca_ala_general_assistance_personal_property_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = MONTH
    label = "Meets property limit for Alameda County General Assistance"
    defined_for = "in_ala"
    reference = "https://www.alamedacountysocialservices.org/acssa-assets/PDF/GA-Policies/GA-Regulations.pdf#page=23"

    def formula(spm_unit, period, parameters):
        p = parameters(
            period
        ).gov.local.ca.ala.general_assistance.personal_property
        personal_property = add(
            spm_unit, period.this_year, ["personal_property"]
        )
        return personal_property <= p.limit



from policyengine_us.model_api import *


class ca_care_income_eligible(Variable):
    value_type = bool
    entity = Household
    definition_period = YEAR
    label = "Eligible for California CARE program by virtue of income"
    documentation = "Eligible for California Alternate Rates for Energy"
    reference = "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?lawCode=PUC&sectionNum=739.1"
    defined_for = StateCode.CA

    def formula(household, period, parameters):
        # Check income eligibility with respect to percent of the poverty line.
        income = household("household_market_income", period)
        p = parameters(period).gov.states.ca.cpuc.care.eligibility
        # Calculate income limit based on federal poverty line, adjusted to
        # set one-person households as two-person.
        poverty_line = household("ca_care_poverty_line", period)
        return income <= (poverty_line * p.fpl_limit)



from policyengine_us.model_api import *


class taxable_retirement_distributions(Variable):
    value_type = float
    entity = Person
    label = "Taxable retirement account distributions"
    unit = USD
    definition_period = YEAR
    adds = [
        "taxable_ira_distributions",
        "taxable_401k_distributions",
        "taxable_sep_distributions",
        "taxable_403b_distributions",
        "keogh_distributions",
    ]



from policyengine_us.model_api import *


class household_local_benefits(Variable):
    value_type = float
    entity = Household
    label = "Household local benefits"
    unit = USD
    documentation = "Benefits paid by local agencies."
    definition_period = YEAR
    adds = "gov.household.household_local_benefits"



from policyengine_us.model_api import *


class co_pension_subtraction_indv(Variable):
    value_type = float
    entity = Person
    label = "Colorado pension and annuity subtraction for eligible individuals"
    defined_for = "co_pension_subtraction_indv_eligible"
    unit = USD
    reference = (
        "https://tax.colorado.gov/sites/tax/files/documents/DR0104AD_2022.pdf#page=1",
        "https://tax.colorado.gov/sites/tax/files/documents/DR_104_Book_2022.pdf#page=12",
        "https://law.justia.com/codes/colorado/2022/title-39/article-22/part-1/section-39-22-104/",
        # C.R.S. 39-22-104(4)(g)(III)
    )
    definition_period = YEAR

    def formula(person, period, parameters):
        p = parameters(period).gov.states.co.tax.income.subtractions.pension
        taxable_pension_income = person(
            "co_pension_subtraction_income", period
        )
        pension_survivors = person("pension_survivors", period)
        co_social_security_subtraction = person(
            "co_social_security_subtraction_indv", period
        )
        age = person("age", period)
        # The maximum subtarction amount is reduced by the social security subtraction amount
        reduced_older_cap = max_(
            p.cap.older - co_social_security_subtraction, 0
        )
        reduced_younger_cap = max_(
            p.cap.younger - co_social_security_subtraction, 0
        )

        capped_older_amount = min_(reduced_older_cap, taxable_pension_income)
        capped_middle_amount = min_(
            reduced_younger_cap, taxable_pension_income
        )
        capped_younger_amount = min_(reduced_younger_cap, pension_survivors)

        return select(
            [
                age >= p.age_threshold.older,
                age >= p.age_threshold.younger,
            ],
            [capped_older_amount, capped_middle_amount],
            default=capped_younger_amount,
        )



from policyengine_us.model_api import *


class person_id(Variable):
    value_type = int
    entity = Person
    label = "Unique reference for this person"
    definition_period = YEAR

    def formula(person, period, parameters):
        return np.arange(len(person("age", period)))



from policyengine_us.model_api import *


class or_disabled_child_dependent_exemptions(Variable):
    value_type = int
    entity = TaxUnit
    label = "OR disabled child dependent exemptions"
    definition_period = YEAR
    reference = (
        "https://www.oregon.gov/dor/forms/FormsPubs/form-or-40-inst_101-040-1_2021.pdf#page=17"
        "https://www.oregonlegislature.gov/bills_laws/ors/ors316.html"  # Subsection 316.099(3)
    )
    defined_for = StateCode.OR

    def formula(tax_unit, period, parameters):
        # Identify qualifying tax unit members.
        person = tax_unit.members
        disabled = person("is_disabled", period)
        # Law references IRC Section 152, which defines child dependent for EITC.
        eitc_qualifying_child = person("is_child_dependent", period)
        # Determine eligibility from AGI.
        federal_agi = tax_unit("adjusted_gross_income", period)
        p = parameters(period).gov.states["or"].tax.income.credits.exemption
        qualifies = federal_agi <= p.income_limit.disabled_child_dependent
        return qualifies * tax_unit.sum(eitc_qualifying_child & disabled)



from policyengine_us.model_api import *


class oh_joint_filing_credit_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Eligible for the Ohio joint filing credit"
    definition_period = YEAR
    reference = "https://codes.ohio.gov/ohio-revised-code/section-5747.05"
    defined_for = StateCode.OH

    def formula(tax_unit, period, parameters):
        filing_status = tax_unit("filing_status", period)
        person = tax_unit.members
        is_head = person("is_tax_unit_head", period)
        is_spouse = person("is_tax_unit_spouse", period)
        p = parameters(period).gov.states.oh.tax.income.credits.joint_filing
        income = person("oh_joint_filing_credit_qualifying_income", period)
        # The head and the spouse have to meet the income requirement individually
        income_eligible = income >= p.income_threshold
        head_eligible = tax_unit.any(is_head & income_eligible)

        spouse_eligible = tax_unit.any(is_spouse & income_eligible)
        # The head and the spouse have to meet the income requirement individually
        head_and_spouse_eligible = head_eligible & spouse_eligible

        status_eligible = filing_status == filing_status.possible_values.JOINT
        return head_and_spouse_eligible & status_eligible



from policyengine_us.model_api import *


class nj_property_tax_deduction_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "New Jersey property tax deduction eligibility"
    definition_period = YEAR
    defined_for = StateCode.NJ

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.nj.tax.income

        filing_status = tax_unit("filing_status", period)
        nj_agi = tax_unit("nj_agi", period)
        income_eligible = nj_agi > p.filing_threshold[filing_status]

        pays_ptax = add(tax_unit, period, ["real_estate_taxes"]) > 0
        pays_rent = tax_unit("rents", period)

        return income_eligible & (pays_ptax | pays_rent)



from policyengine_us.model_api import *


class ctc_maximum_with_arpa_addition(Variable):
    value_type = float
    entity = TaxUnit
    label = "Maximum CTC for ARPA"
    unit = USD
    documentation = (
        "Maximum value of the Child Tax Credit, before phase-out, under ARPA."
    )
    definition_period = YEAR

    adds = ["ctc_maximum", "ctc_arpa_addition"]



from policyengine_us.model_api import *


class tax_exempt_sep_distributions(Variable):
    value_type = float
    entity = Person
    label = "tax-exempt SEP distributions"
    unit = USD
    definition_period = YEAR



from policyengine_us.model_api import *


class is_demographic_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = MONTH
    label = "Demographic eligibility for TANF"
    documentation = "Whether any person in a family applying for the Temporary Assistance for Needy Families program meets demographic requirements."
    adds = ["is_person_demographic_tanf_eligible"]



from policyengine_us.model_api import *


class nc_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "North Carolina TANF countable earned income"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.NC

    adds = "gov.states.nc.ncdhhs.tanf.income.earned"



from policyengine_us.model_api import *


class earned_income_last_year(Variable):
    value_type = float
    entity = Person
    label = "Earned income last year"
    unit = USD
    documentation = "Prior-year income from wages or self-employment"
    definition_period = YEAR

    adds = ["employment_income_last_year", "self_employment_income_last_year"]



from policyengine_us.model_api import *


class healthcare_benefit_value(Variable):
    value_type = float
    label = "Cash equivalent of health coverage"
    entity = Household
    definition_period = YEAR
    unit = USD
    adds = ["medicaid_cost", "per_capita_chip", "aca_ptc"]



from policyengine_us.model_api import *


class prior_energy_efficient_window_credits(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Prior years energy efficient window credit"
    documentation = "Energy efficient window credits claimed in prior years"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/25C#b_2"



from policyengine_us.model_api import *


class tax_exempt_social_security(Variable):
    value_type = float
    entity = TaxUnit
    label = "Tax-exempt Social Security"
    unit = USD
    definition_period = YEAR

    adds = ["tax_unit_social_security"]
    subtracts = ["tax_unit_taxable_social_security"]



from policyengine_us.model_api import *


class md_deductions(Variable):
    value_type = float
    entity = TaxUnit
    label = "MD deductions"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://govt.westlaw.com/mdc/Document/N05479690A64A11DBB5DDAC3692B918BC?viewType=FullText&originationContext=documenttoc&transitionType=CategoryPageItem&contextData=(sc.Default)"
        "https://www.marylandtaxes.gov/forms/21_forms/Resident_Booklet.pdf#page=5"
        "https://www.marylandtaxes.gov/forms/22_forms/Resident_Booklet.pdf#page=5"
    )
    defined_for = StateCode.MD

    def formula(tax_unit, period, parameters):
        md_itmded = tax_unit("md_itemized_deductions", period)
        md_stdded = tax_unit("md_standard_deduction", period)
        # 2021 and 2022 Form 502 instructions on page 5 include this FAQ:
        # 3. Itemized deductions.
        #    Q: Can I claim itemized deductions on my Maryland return if
        #       I claimed standard deduction on my federal return?
        #    A: No. You may claim itemized deductions on your Maryland
        #       return only if you claimed itemized deductions on your
        #       federal return. If you claimed your itemized deductions
        #       on your federal return, you may figure your tax using
        #       both deduction methods to determine which is best for you.
        federal_itemizer = tax_unit("tax_unit_itemizes", period)
        return where(federal_itemizer, max_(md_stdded, md_itmded), md_stdded)



from policyengine_us.model_api import *


class capital_gains(Variable):
    value_type = float
    entity = Person
    label = "Capital gains (both short-term and long-term)"
    unit = USD
    documentation = "Net gain from disposition of property."
    definition_period = YEAR
    adds = ["short_term_capital_gains", "long_term_capital_gains"]



from policyengine_us.model_api import *


class de_itemized_deductions_joint(Variable):
    value_type = float
    entity = Person
    label = "Delaware itemized deductions when married filing jointly"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://revenuefiles.delaware.gov/2022/TY22_PIT-RSA_2022-02_PaperInteractive.pdf",  # § 1109
        "https://delcode.delaware.gov/title30/c011/sc02/index.html",
    )
    defined_for = StateCode.DE

    def formula(person, period, parameters):
        deductions = person.tax_unit("de_itemized_deductions_unit", period)
        is_head = person("is_tax_unit_head", period)
        return deductions * is_head



from policyengine_us.model_api import *


class ca_exemptions(Variable):
    value_type = float
    entity = TaxUnit
    label = "CA Exemptions"
    defined_for = StateCode.CA
    unit = USD
    definition_period = YEAR
    reference = "https://www.ftb.ca.gov/forms/2021/2021-540.pdf"

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.ca.tax.income.exemptions
        agi = tax_unit("adjusted_gross_income", period)
        filing_status = tax_unit("filing_status", period)

        # calculating phase out amount per credit
        over_agi_threshold = max_(0, agi - p.phase_out.start[filing_status])
        increments = np.ceil(
            over_agi_threshold / p.phase_out.increment[filing_status]
        )
        exemption_reduction = increments * p.phase_out.amount

        # Personal Exemptions
        personal_exemption_count = p.personal_scale[filing_status]
        personal_aged_blind_exemption_count = (
            personal_exemption_count + tax_unit("aged_blind_count", period)
        )
        personal_aged_blind_exemption = max_(
            0,
            personal_aged_blind_exemption_count
            * (p.amount - exemption_reduction),
        )

        # Dependent exemptions
        dependents = tax_unit("tax_unit_dependents", period)
        dependent_exemptions = max_(
            0, dependents * (p.dependent_amount - exemption_reduction)
        )

        # total exemptions
        return personal_aged_blind_exemption + dependent_exemptions



from policyengine_us.model_api import *


class mo_income_tax_before_credits(Variable):
    value_type = float
    entity = Person
    label = "Missouri income tax before credits"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://dor.mo.gov/forms/MO-1040%20Print%20Only_2021.pdf",
        "https://www.revisor.mo.gov/main/OneChapter.aspx?chapter=143",
    )
    defined_for = StateCode.MO

    def formula(person, period, parameters):
        taxable_income = person("mo_taxable_income", period)
        rates = parameters(period).gov.states.mo.tax.income.rates
        exempt = person("mo_income_tax_exempt", period)
        return where(exempt, 0, rates.calc(taxable_income))



from policyengine_us.model_api import *


class nj_retirement_exclusion_fraction(Variable):
    value_type = float
    entity = TaxUnit
    label = "New Jersey retirement exclusion fraction based on total income"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.state.nj.us/treasury/taxation/pdf/current/1040i.pdf#page=21",
        "https://law.justia.com/codes/new-jersey/2022/title-54a/section-54a-6-10/",
    )
    defined_for = StateCode.NJ

    def formula(tax_unit, period, parameters):
        total_income = add(tax_unit, period, ["nj_total_income"])
        filing_status = tax_unit("filing_status", period)
        status = filing_status.possible_values
        p = parameters(
            period
        ).gov.states.nj.tax.income.exclusions.retirement.pension
        return select(
            [
                filing_status == status.SINGLE,
                filing_status == status.JOINT,
                filing_status == status.HEAD_OF_HOUSEHOLD,
                filing_status == status.SURVIVING_SPOUSE,
                filing_status == status.SEPARATE,
            ],
            [
                p.percentage.single.calc(total_income, right=True),
                p.percentage.joint.calc(total_income, right=True),
                p.percentage.head_of_household.calc(total_income, right=True),
                p.percentage.surviving_spouse.calc(total_income, right=True),
                p.percentage.separate.calc(total_income, right=True),
            ],
        )



from policyengine_us.model_api import *


class co_refundable_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Refundable Child Tax Credit replicated to include the Colorado limitations"
    unit = USD
    documentation = (
        "Total value of the refundable portions of the Child Tax Credit."
    )
    definition_period = YEAR
    reference = (
        # C.R.S. 39-22-129. Child tax credit - legislative declaration - definitions.
        "https://casetext.com/statute/colorado-revised-statutes/title-39-taxation/specific-taxes/income-tax/article-22-income-tax/part-1-general/section-39-22-129-child-tax-credit-legislative-declaration-definitions-repeal",
        # 2022 Colorado Child Tax Credit
        "https://tax.colorado.gov/sites/tax/files/documents/DR_0104CN_2022.pdf#page=1",
        # Colorado Individual Income Tax Filing Guide - Instructions for Select Credits from the DR 0104CR - Line 1 Child Tax Credit
        "https://tax.colorado.gov/sites/tax/files/documents/DR_104_Book_2022.pdf#page=16",
    )
    defined_for = StateCode.CO

    def formula(tax_unit, period, parameters):
        # follow 2022 DR 0104CN form and its instructions (in Book cited above):
        adjusted_fed_ctc = tax_unit("co_non_refundable_ctc", period)  # Line 7
        max_child_amount = tax_unit("co_federal_ctc_maximum", period)  # Line 3
        credit_excess_over_tax = max_(
            0, max_child_amount - adjusted_fed_ctc
        )  # Line 8
        p = parameters(period).gov.irs.credits.ctc
        statutory_cap = p.refundable.individual_max  # Line 9
        children = tax_unit("co_ctc_eligible_children_count", period)
        total_statutory_cap = min_(
            statutory_cap * children, credit_excess_over_tax
        )  # Line 10
        earnings = tax_unit("tax_unit_earned_income", period)  # Line 11
        earnings_over_threshold = max_(
            0, earnings - p.refundable.phase_in.threshold
        )  # Line 12
        relevant_earnings = (
            earnings_over_threshold * p.refundable.phase_in.rate
        )  # Line 13
        social_security_tax = tax_unit(
            "ctc_social_security_tax", period
        )  # Line 14 - 16
        federal_eitc = tax_unit("eitc", period)  # Line 17a
        social_security_excess = max_(
            0, social_security_tax - federal_eitc
        )  # Line 18
        tax_increase = where(
            children
            < p.refundable.phase_in.min_children_for_ss_taxes_minus_eitc,
            relevant_earnings,
            max_(relevant_earnings, social_security_excess),
        )  # Line 19
        return min_(total_statutory_cap, tax_increase)  # Line 20



from policyengine_us.model_api import *


class snap_excess_shelter_expense_deduction(Variable):
    value_type = float
    entity = SPMUnit
    definition_period = MONTH
    documentation = (
        "Excess shelter expense deduction for calculating SNAP benefit amount"
    )
    label = "SNAP shelter deduction"
    reference = ("United States Code, Title 7, Section 2014(e)(6)",)
    unit = USD

    def formula(spm_unit, period, parameters):
        p = parameters(
            period
        ).gov.usda.snap.income.deductions.excess_shelter_expense
        # Calculate uncapped shelter deduction as housing costs in excess of
        # income threshold.
        net_income_pre_shelter = spm_unit(
            "snap_net_income_pre_shelter", period
        )
        subtracted_income = p.income_share_disregard * net_income_pre_shelter
        housing_cost = add(
            spm_unit, period, ["snap_utility_allowance", "housing_cost"]
        )
        uncapped_ded = max_(housing_cost - subtracted_income, 0)
        # Calculate capped deduction based on state group parameter.
        state_group = spm_unit.household("snap_region_str", period)
        ded_cap = p.cap[state_group]
        capped_ded = min_(uncapped_ded, ded_cap)
        has_elderly_disabled = spm_unit("has_usda_elderly_disabled", period)
        # Cap for all but elderly/disabled people and add utility allowance.
        non_homeless_shelter_deduction = where(
            has_elderly_disabled, uncapped_ded, capped_ded
        )
        # Homeless shelter deduction is flat and has no utility component.
        state = spm_unit.household("state_code_str", period)
        homeless_deduction = p.homeless.deduction * p.homeless.available[state]
        return where(
            spm_unit.household("is_homeless", period)
            & (housing_cost > 0)
            & (homeless_deduction > non_homeless_shelter_deduction),
            homeless_deduction,
            non_homeless_shelter_deduction,
        )



from policyengine_us.model_api import *


class ct_tuition_subtraction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Connecticut tuition subtraction"
    unit = USD
    definition_period = YEAR
    reference = "https://www.cga.ct.gov/current/pub/chap_229.htm#sec_12-701a"
    defined_for = StateCode.CT

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.ct.tax.income.subtractions.tuition
        filing_status = tax_unit("filing_status", period)
        cap = p.cap[filing_status]
        # Qualified tuition expenses as defined under Section 529(b) of the Internal Revenue Code
        tuition = add(tax_unit, period, ["qualified_tuition_expenses"])
        return min_(tuition, cap)



from policyengine_us.model_api import *


class PellGrantEligibilityType(Enum):
    MAXIMUM = "maximum"
    MINIMUM = "minimum"
    INELIGIBLE = "ineligible"


class pell_grant_eligibility_type(Variable):
    value_type = Enum
    possible_values = PellGrantEligibilityType
    default_value = PellGrantEligibilityType.INELIGIBLE
    entity = Person
    definition_period = YEAR
    label = "Maximum, minimum, or ineligible for Pell Grant"

    def formula(person, period, parameters):
        agi = person.tax_unit("adjusted_gross_income", period)
        # FPG from the prior-prior year.
        fpg = person.tax_unit("tax_unit_fpg", period.offset(-2, "year"))
        fpg_percent = agi / fpg

        max_limit = person("pell_grant_max_fpg_percent_limit", period)
        min_limit = person("pell_grant_min_fpg_percent_limit", period)

        return select(
            [fpg_percent <= max_limit, fpg_percent <= min_limit],
            [
                PellGrantEligibilityType.MAXIMUM,
                PellGrantEligibilityType.MINIMUM,
            ],
            default=PellGrantEligibilityType.INELIGIBLE,
        )



from policyengine_us.model_api import *


class mi_standard_deduction_tier_three(Variable):
    value_type = float
    entity = TaxUnit
    label = "Michigan tier three standard deduction"
    unit = USD
    definition_period = YEAR
    reference = (
        "http://legislature.mi.gov/doc.aspx?mcl-206-30",  # (9)(e)
        "https://www.michigan.gov/taxes/iit/retirement-and-pension-benefits/michigan-standard-deduction",
        "https://www.michigan.gov/taxes/-/media/Project/Websites/taxes/Forms/2022/2022-IIT-Forms/BOOK_MI-1040.pdf#page=16",
    )
    defined_for = "mi_standard_deduction_tier_three_eligible"

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.mi.tax.income.deductions
        # Line 1: enter base amount, based on filing status
        filing_status = tax_unit("filing_status", period)
        base_amount = p.standard.tier_three.amount[filing_status]

        # Exemption(s), taxable Social Security benefits,
        # military compensation (including retirement benefits),
        # Michigan National Guard retirement benefits and railroad
        # retirement benefits included in AGI may reduce the amount
        # eligible to be claimed on this deduction.

        person = tax_unit.members
        is_head_or_spouse = person("is_tax_unit_head_or_spouse", period)
        # Line 2: enter military retirement pay
        military_retirement_pay = person("military_retirement_pay", period)
        # Line 3: enter military service income or taxable social security
        taxable_ss = person("taxable_social_security", period)
        military_service_income = person("military_service_income", period)
        larger_ss_or_military_pay = max_(taxable_ss, military_service_income)
        # Line 4 are personal (and stillborn) exemptions
        mi_personal_exemptions = tax_unit("mi_personal_exemptions", period)
        # Line 5: add lines 2 through 4
        total_person_reductions = tax_unit.sum(
            is_head_or_spouse
            * (military_retirement_pay + larger_ss_or_military_pay)
        )
        total_reductions = total_person_reductions + mi_personal_exemptions
        # Line 6: subtract line 5 from line 1
        standard_deduction_tier_three = max_(base_amount - total_reductions, 0)
        # Worksheet 3.3: Retirement and Pension Benefits Subtraction for Section D of Form 4884
        if p.retirement_benefits.expanded.availability:
            expanded_retirement_benefits_deduction = tax_unit(
                "mi_expanded_retirement_benefits_deduction",
                period,
            )
            return max_(
                standard_deduction_tier_three,
                expanded_retirement_benefits_deduction,
            )
        else:
            return standard_deduction_tier_three



from policyengine_us.model_api import *


class is_grandparent_of_filer_or_spouse(Variable):
    value_type = bool
    entity = Person
    label = "Is a grandparent of the filer or spouse"
    definition_period = YEAR



from policyengine_us.model_api import *


class vt_personal_exemptions(Variable):
    value_type = float
    entity = TaxUnit
    label = "Vermont personal exemptions"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.VT
    reference = (
        "https://tax.vermont.gov/sites/tax/files/documents/IN-111-2022.pdf"
    )

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.vt.tax.income.exemption
        is_joint = tax_unit("tax_unit_is_joint", period)
        elsewhere_head = tax_unit("head_is_dependent_elsewhere", period)
        elsewhere_spouse = tax_unit("spouse_is_dependent_elsewhere", period)
        eligible_head = (~elsewhere_head).astype(int)
        eligible_spouse = (~elsewhere_spouse).astype(int)
        eligible_count = eligible_head + (eligible_spouse * is_joint)
        # add number of other dependents claimed on federal Form 1040 (line 5c)
        dependents = tax_unit("tax_unit_count_dependents", period)
        total_exemption_count = eligible_count + dependents
        return total_exemption_count * p.personal



from policyengine_us.model_api import *


class person_weight(Variable):
    value_type = float
    entity = Person
    label = "Person weight"
    definition_period = YEAR
    uprating = "calibration.gov.census.populations.total"

    def formula(person, period, parameters):
        return person.household("household_weight", period)



from policyengine_us.model_api import *


class co_cdcc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Colorado Child Care Expenses Credit"
    unit = USD
    documentation = (
        "https://advance.lexis.com/documentpage/?pdmfid=1000516&crid=d14880b7-7410-4295-bcf1-2e099e57d8f3&pdistocdocslideraccess=true&config=014FJAAyNGJkY2Y4Zi1mNjgyLTRkN2YtYmE4OS03NTYzNzYzOTg0OGEKAFBvZENhdGFsb2d592qv2Kywlf8caKqYROP5&pddocfullpath=%2Fshared%2Fdocument%2Fstatutes-legislation%2Furn%3AcontentItem%3A65HV-06G3-CGX8-050B-00008-00&pdcomponentid=234177&pdtocnodeidentifier=ABPAACAACAABAACABA&ecomp=k2vckkk&prid=e2e32763-f8fa-4832-8191-f70124d877f6"
        "https://tax.colorado.gov/sites/tax/files/documents/DR_104_Book_2022.pdf#page=46"
    )
    definition_period = YEAR
    defined_for = StateCode.CO

    def formula(tax_unit, period, parameters):
        # follow 2022 DR 0347 form and its instructions (in Book cited above):
        p = parameters(period).gov.states.co.tax.income.credits
        fed_agi = tax_unit("adjusted_gross_income", period)  # Line 4
        # calculate regular Colorado CDCC in Part III
        capped_fed_cdcc = tax_unit("capped_cdcc", period)  # Line 8
        match_rate = p.cdcc.match.calc(fed_agi, right=True)
        return capped_fed_cdcc * match_rate  # Line 9
        # calculate low-income Colorado CDCC in Part IV in co_low_income_cdcc



from policyengine_us.model_api import *


class mn_marriage_credit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Minnesota marriage credit"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.revenue.state.mn.us/sites/default/files/2023-01/m1ma_21.pdf"
        "https://www.revenue.state.mn.us/sites/default/files/2023-01/m1ma_22.pdf"
    )
    defined_for = StateCode.MN

    def formula(tax_unit, period, parameters):
        mn_itax = parameters(period).gov.states.mn.tax.income
        p = mn_itax.credits.marriage
        # determine filing status eligibility
        filing_status = tax_unit("filing_status", period)
        joint = filing_status == filing_status.possible_values.JOINT
        # determine individual income eligibility
        person = tax_unit.members
        income = add(person, period, p.income_sources) - person(
            "self_employment_tax_ald_person", period
        )
        is_head = person("is_tax_unit_head", period)
        head_income = tax_unit.sum(is_head * income)
        is_spouse = person("is_tax_unit_spouse", period)
        spouse_income = tax_unit.sum(is_spouse * income)
        min_income = min_(head_income, spouse_income)
        individual_income_eligible = min_income >= p.minimum_individual_income
        # determine taxable income eligibility
        taxinc0 = tax_unit("mn_taxable_income", period)
        taxable_income_eligible = taxinc0 >= p.minimum_taxable_income
        # determine overall eligibility for credit
        eligible = joint & individual_income_eligible & taxable_income_eligible
        # determine credit amount
        std_ded = mn_itax.deductions.standard.base[filing_status]
        fractional_std_ded = p.standard_deduction_fraction * std_ded
        taxinc1 = max_(0, min_income - fractional_std_ded)
        itax1 = mn_itax.rates.single.calc(taxinc1)
        taxinc2 = max_(0, taxinc0 - taxinc1)
        itax2 = mn_itax.rates.single.calc(taxinc2)
        itax0 = tax_unit("mn_basic_tax", period)
        amount = max_(0, itax0 - itax1 - itax2)
        # capped credit only for eligibles
        return eligible * min_(amount, p.maximum_amount)



from policyengine_us.model_api import *


class ut_at_home_parent_credit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Utah at-home parent credit"
    unit = USD
    definition_period = YEAR
    defined_for = "ut_at_home_parent_credit_agi_eligible"
    reference = (
        "https://le.utah.gov/xcode/Title59/Chapter10/59-10-S1005.html",
        "https://www.taxformfinder.org/forms/2021/2021-utah-tc-40-full-packet.pdf#page=23",
    )

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        age = person("age", period)
        is_dependent = person("is_tax_unit_dependent", period)
        p = parameters(period).gov.states.ut.tax.income.credits.at_home_parent
        qualifying_child = (age < p.max_child_age) & is_dependent
        count_qualifying_children = tax_unit.sum(qualifying_child)

        # Multiply by each qualifying parent; they can claim it separately.
        income_eligible_person = add(
            tax_unit,
            period,
            ["ut_at_home_parent_credit_earned_income_eligible_person"],
        )

        return p.amount * count_qualifying_children * income_eligible_person



from policyengine_us.model_api import *


class broadband_cost_after_lifeline(Variable):
    value_type = float
    entity = SPMUnit
    label = "Broadband costs after Lifeline"
    documentation = "Broadband costs after Lifeline benefits"
    definition_period = YEAR
    unit = USD
    reference = "https://www.law.cornell.edu/cfr/text/47/54.403"

    def formula(spm_unit, period, parameters):
        broadband = spm_unit("broadband_cost", period)
        lifeline = spm_unit("lifeline", period)
        return max_(broadband - lifeline, 0)



from policyengine_us.model_api import *


class household_net_income_including_health_benefits(Variable):
    value_type = float
    entity = Household
    label = "Net income including health benefits"
    definition_period = YEAR
    unit = USD
    adds = ["household_net_income", "healthcare_benefit_value"]



from policyengine_us.model_api import *


class al_dependent_exemption(Variable):
    value_type = float
    entity = TaxUnit
    label = "Alabama dependent exemption"
    unit = USD
    # The Code of Alabama 1975 Section 40-18-19 (a)(9).
    documentation = "https://alisondb.legislature.state.al.us/alison/CodeOfAlabama/1975/Coatoc.htm"
    definition_period = YEAR
    defined_for = StateCode.AL

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.al.tax.income.exemptions
        al_agi = tax_unit("al_agi", period)
        dependents = tax_unit("tax_unit_dependents", period)
        exemption_per_dependent = p.dependent.calc(al_agi)
        return dependents * exemption_per_dependent



from policyengine_us.model_api import *


class wa_tanf_countable_resources(Variable):
    value_type = float
    entity = SPMUnit
    label = "Countable resources for Washington TANF"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.WA



from policyengine_us.model_api import *


class has_usda_elderly_disabled(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = YEAR
    documentation = "Whether the SPM unit has a person who meets USDA definitions of elderly or disabled"
    label = "Has USDA elderly or disabled people"

    def formula(spm_unit, period, parameters):
        person = spm_unit.members
        elderly = person("is_usda_elderly", period)
        disabled = person("is_usda_disabled", period)
        return spm_unit.any(elderly | disabled)



from policyengine_us.model_api import *


class childcare_expenses(Variable):
    value_type = float
    entity = SPMUnit
    label = "Child care expenses"
    definition_period = YEAR
    unit = USD

    def formula(spm_unit, period, parameters):
        pre_subsidy_childcare_expenses = add(
            spm_unit, period, ["pre_subsidy_childcare_expenses"]
        )
        # States where we model childcare subsidies.
        STATES_WITH_CHILD_CARE_SUBSIDIES = ["CA", "CO", "NE"]
        subsidy_variables = [
            i.lower() + "_child_care_subsidies"
            for i in STATES_WITH_CHILD_CARE_SUBSIDIES
        ]
        subsidies = add(spm_unit, period, subsidy_variables)
        return max_(pre_subsidy_childcare_expenses - subsidies, 0)



from policyengine_us.model_api import *


class hud_hap(Variable):
    value_type = float
    entity = SPMUnit
    label = "HUD housing assistance payment"
    unit = USD
    documentation = "HUD housing assistance payment"
    definition_period = YEAR
    reference = "https://www.hud.gov/sites/dfiles/PIH/documents/HCV_Guidebook_Calculating_Rent_and_HAP_Payments.pdf"

    def formula(spm_unit, period, parameters):
        max_subsidy = spm_unit("hud_max_subsidy", period)
        ttp = spm_unit("hud_ttp", period)
        gross_rent = spm_unit("hud_gross_rent", period)
        rent_minus_ttp = max_(0, gross_rent - ttp)
        return min_(max_subsidy, rent_minus_ttp)



from policyengine_us.model_api import *


class mi_standard_deduction_tier_three_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Eligible for the Michigan standard deduction"
    definition_period = YEAR
    reference = (
        "http://legislature.mi.gov/doc.aspx?mcl-206-30",  # (9)(e)
        "https://www.michigan.gov/taxes/iit/retirement-and-pension-benefits/michigan-standard-deduction",
        "https://www.michigan.gov/taxes/-/media/Project/Websites/taxes/Forms/2022/2022-IIT-Forms/BOOK_MI-1040.pdf#page=16",
    )
    defined_for = StateCode.MI

    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.mi.tax.income.deductions.standard.tier_three

        older_spouse_birth_year = tax_unit("older_spouse_birth_year", period)

        return p.birth_year.calc(older_spouse_birth_year)



from policyengine_us.model_api import *


class mt_personal_exemptions_indiv(Variable):
    value_type = float
    entity = Person
    label = "Montana exemptions when married couples file separately"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MT

    def formula(person, period, parameters):
        p = parameters(period).gov.states.mt.tax.income.exemptions

        if p.applies:
            head_or_spouse = person("is_tax_unit_head_or_spouse", period)
            blind = person("is_blind", period)
            blind_head_or_spouse = blind * head_or_spouse
            # Allocate the dependent exemption to the head
            head = person("is_tax_unit_head", period)
            aged_exemption = person(
                "mt_aged_exemption_eligible_person", period
            )
            exemption_count = (
                head_or_spouse.astype(int)
                + blind_head_or_spouse.astype(int)
                + aged_exemption.astype(int)
            )
            return exemption_count * p.amount

        return 0



from policyengine_core.reforms import Reform
from policyengine_us.model_api import *


class pa_tax_forgiveness_rate(Variable):
    value_type = float
    entity = TaxUnit
    label = "PA tax forgiveness on eligibility income"
    unit = "/1"
    definition_period = YEAR
    reference = "https://www.revenue.pa.gov/FormsandPublications/FormsforIndividuals/PIT/Documents/2021/2021_pa-40in.pdf#page=39"
    defined_for = StateCode.PA
    """
    TAXSIM erroneously phases in tax forgiveness smoothly.
    """

    def formula(tax_unit, period, parameters):
        eligibility_income = tax_unit("pa_eligibility_income", period)
        person = tax_unit.members
        is_child_dependent = person("is_child_of_tax_head", period) & person(
            "is_tax_unit_dependent", period
        )
        child_dependents = tax_unit.sum(is_child_dependent)
        # filing status affects the base, where it doubles for married claimants
        filing_status = tax_unit("filing_status", period)
        filing_statuses = filing_status.possible_values
        joint_separate = (filing_status == filing_statuses.JOINT) | (
            filing_status == filing_statuses.SEPARATE
        )
        base_multiplier = where(joint_separate, 2, 1)
        p = parameters(period).gov.states.pa.tax.income.forgiveness
        base = p.base * base_multiplier
        rate_per_dependent = p.dependent_rate
        eligibility_income_increment = base + (
            rate_per_dependent * child_dependents
        )
        excess = eligibility_income - eligibility_income_increment
        forgiveness_increment = p.rate_increment
        increments = excess / forgiveness_increment
        percent = p.tax_back
        return min_(max_(1 - percent * increments, 0), 1)


class ctc_refundable_maximum(Variable):
    value_type = float
    entity = TaxUnit
    label = "Maximum refundable CTC"
    unit = USD
    documentation = "The maximum refundable CTC for this person."
    definition_period = YEAR
    reference = (
        "https://www.law.cornell.edu/uscode/text/26/24#a",
        "https://www.law.cornell.edu/uscode/text/26/24#h",
        "https://www.law.cornell.edu/uscode/text/26/24#i",
        "https://www.irs.gov/pub/irs-prior/f1040--2021.pdf",
        "https://www.irs.gov/pub/irs-prior/f1040s8--2021.pdf",
    )
    """
    TAXSIM erroneously makes the adult CTC refundable as well.
    """

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        # Use either normal or ARPA CTC maximums.
        child_amount = max_(
            person("ctc_child_individual_maximum", period),
            person("ctc_child_individual_maximum_arpa", period),
        )
        adult_amount = person("ctc_adult_individual_maximum", period)
        ctc = parameters(period).gov.irs.credits.ctc
        if ctc.refundable.fully_refundable:
            # Fully refundable CTC does not affect the adult CTC.
            return tax_unit.sum(child_amount + adult_amount)
        return tax_unit.sum(min_(child_amount, ctc.refundable.individual_max))


class ctc_arpa_uncapped_phase_out(Variable):
    value_type = float
    entity = TaxUnit
    label = "Uncapped phase-out of ARPA CTC increase"
    unit = USD
    definition_period = YEAR
    """
    TAXSIM erroneously phases out the CTC smoothly rather than in increments.
    """

    def formula(tax_unit, period, parameters):
        # Logic sequence follows the form, which is clearer than the IRC.
        p = parameters(period).gov.irs.credits.ctc.phase_out.arpa
        # defined_for didn't work.
        if not p.in_effect:
            return 0
        # The ARPA CTC has two phase-outs: the original, and a new phase-out
        # applying before and only to the increase in the maximum CTC under ARPA.
        # Calculate the income used to assess the new phase-out.
        threshold = tax_unit("ctc_arpa_phase_out_threshold", period)
        agi = tax_unit("adjusted_gross_income", period)
        excess = max_(0, agi - threshold)
        return excess * p.amount / p.increment


class ctc_phase_out(Variable):
    value_type = float
    entity = TaxUnit
    label = "CTC reduction from income"
    unit = USD
    documentation = "Reduction of the total CTC due to income."
    definition_period = YEAR
    """
    TAXSIM erroneously phases out the CTC smoothly rather than in increments.
    """

    def formula(tax_unit, period, parameters):
        # TCJA's phase-out changes are purely parametric so don't require
        # structural reform.

        # The ARPA CTC has two phase-outs: the original, and a new phase-out
        # applying before and only to the increase in the maximum CTC under ARPA.

        # Start with the normal phase-out.
        income = tax_unit("adjusted_gross_income", period)
        p = parameters(period).gov.irs.credits.ctc.phase_out
        phase_out_threshold = tax_unit("ctc_phase_out_threshold", period)
        excess = max_(0, income - phase_out_threshold)
        return excess * p.amount / p.increment


class taxsim(Reform):
    def apply(self):
        self.update_variable(pa_tax_forgiveness_rate)
        self.update_variable(ctc_refundable_maximum)
        self.update_variable(ctc_arpa_uncapped_phase_out)
        self.update_variable(ctc_phase_out)



from policyengine_us.model_api import *


class non_refundable_american_opportunity_credit_potential(Variable):
    value_type = float
    entity = TaxUnit
    label = "Potential value of the Non-refundable American Opportunity Credit"
    unit = USD
    documentation = "Value of the non-refundable portion of the American Opportunity Credit"
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/25A#i"

    adds = ["american_opportunity_credit"]
    subtracts = ["refundable_american_opportunity_credit"]



from policyengine_us.model_api import *


class snap_child_support_gross_income_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "SNAP child support payment deduction from gross income"
    unit = USD
    documentation = (
        "Deduction for child support payments when computing SNAP gross income"
    )
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/uscode/text/7/2014#e_4"

    def formula(spm_unit, period, parameters):
        child_support = add(spm_unit, period, ["child_support_expense"])
        state = spm_unit.household("state_code_str", period)
        is_deductible = parameters(
            period
        ).gov.usda.snap.income.deductions.child_support[state]
        return is_deductible * child_support



from policyengine_us.model_api import *


class il_aabd_personal_allowance(Variable):
    value_type = float
    entity = Person
    label = (
        "Illinois Aid to the Aged, Blind or Disabled (AABD) personal allowance"
    )
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.IL
    reference = "https://www.dhs.state.il.us/page.aspx?item=15913"

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.states.il.dhs.aabd.payment.personal_allowance
        size = person.spm_unit("spm_unit_size", period)
        capped_size = clip(size, 1, 8)
        is_bedfast = person("il_aabd_is_bedfast", period)
        return where(is_bedfast, p.bedfast[capped_size], p.active[capped_size])



from policyengine_us.model_api import *


class household_net_income(Variable):
    value_type = float
    entity = Household
    label = "net income"
    definition_period = YEAR
    unit = USD
    adds = [
        "household_market_income",
        "household_benefits",
        "household_refundable_tax_credits",
    ]
    subtracts = ["household_tax_before_refundable_credits"]



from policyengine_us.model_api import *


class federal_eitc_without_age_minimum(Variable):
    value_type = float
    entity = TaxUnit
    label = "Federal EITC without age minimum"
    unit = USD
    documentation = "The federal EITC with the minimum age condition ignored."
    definition_period = YEAR
    reference = "https://casetext.com/statute/code-of-maryland/article-tax-general/title-10-income-tax/subtitle-7-income-tax-credits/section-10-704-effective-until-6302023-for-earned-income"
    defined_for = StateCode.MD

    def formula(tax_unit, period, parameters):
        # set up simulation clone
        simulation = tax_unit.simulation
        simulation.max_spiral_loops = 10
        simulation._check_for_cycle = lambda *args: None
        EITC_VARIABLES = [
            "eitc_agi_limit",
            "eitc_child_count",
            "eitc_eligible",
            "eitc_demographic_eligible",
            "eitc_investment_income_eligible",
            "eitc_maximum",
            "eitc_phased_in",
            "eitc_reduction",
            "eitc",
        ]
        for variable in EITC_VARIABLES:
            simulation.get_holder(variable).delete_arrays()

        # modify EITC minimum age condition in simulation clone
        tbs = simulation.tax_benefit_system
        original_age = tbs.parameters.gov.irs.credits.eitc.eligibility.age.min(
            period
        )
        tbs.parameters.gov.irs.credits.eitc.eligibility.age.min.update(
            value=0,
            period=period,
        )
        tbs.parameters.gov.irs.credits.eitc._at_instant_cache = {}
        eitc = simulation.calculate("eitc", period)
        for variable in EITC_VARIABLES:
            simulation.get_holder(variable).delete_arrays()
        tbs.parameters.gov.irs.credits.eitc.eligibility.age.min.update(
            value=original_age,
            period=period,
        )
        tbs.parameters.gov.irs.credits.eitc._at_instant_cache = {}
        return eitc



from policyengine_us.model_api import *


def _apply_ssi_exclusions(
    earned_income: ArrayLike,
    unearned_income: ArrayLike,
    parameters: ParameterNode,
    period: Period,
) -> ArrayLike:
    """
    Applies standard SSI income exclusions to convert total earned/unearned income
    into countable income. This follows 20 CFR §§ 416.1112 and 416.1124:
      1) Subtract $20 'general' exclusion from unearned (or from earned if unearned < 20).
      2) Subtract $65 'earned' exclusion from any remaining earned income.
      3) Exclude 50% of the remainder of earned income.

    All amounts are stored as annual but exclusions are monthly amounts, so we convert
    from annual -> monthly, apply monthly exclusions, then go back to annual.
    """
    # Convert from annual to monthly
    earned_monthly = earned_income / MONTHS_IN_YEAR
    unearned_monthly = unearned_income / MONTHS_IN_YEAR

    p = parameters(period).gov.ssa.ssi.income.exclusions

    # Step 1: Subtract general exclusion from unearned first
    applied_general = min_(p.general, unearned_monthly)
    countable_unearned = unearned_monthly - applied_general

    # The remainder of the general exclusion (if unearned < 20) applies to earned
    leftover_general = p.general - applied_general
    total_earned_excl = p.earned + leftover_general

    # Step 2 & 3: Apply the total earned exclusion, then exclude 50% of the rest
    earned_after_flat = max_(earned_monthly - total_earned_excl, 0)
    # The 'earned_share' parameter is the fraction excluded above the flat. Typically 0.5 => 50% excluded.
    # So the fraction that remains is (1 - earned_share).
    countable_earned = earned_after_flat * (1.0 - p.earned_share)

    # Convert back to annual
    return (countable_unearned + countable_earned) * MONTHS_IN_YEAR



from policyengine_us.model_api import *


class id_income_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "Idaho income tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.ID
    adds = ["id_income_tax_before_refundable_credits", "id_pbf"]
    subtracts = ["id_refundable_credits"]



from policyengine_us.model_api import *


class is_homeless(Variable):
    value_type = bool
    entity = Household
    definition_period = YEAR
    documentation = "Whether all members are homeless individuals and are not receiving free shelter throughout the month"
    label = "Is homeless"



from policyengine_us.model_api import *


class tax_unit_household_id(Variable):
    value_type = float
    entity = TaxUnit
    label = "Tax unit household ID"
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        return tax_unit.value_from_first_person(
            person.household("household_id", period)
        )



from policyengine_us.model_api import *


class current_pregnancy_month(Variable):
    value_type = int
    entity = Person
    label = "Current pregnancy month"
    definition_period = MONTH
    defined_for = "is_pregnant"



from policyengine_us.model_api import *


class ri_retirement_income_subtraction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Rhode Island retirement income subtraction"
    unit = USD
    definition_period = YEAR
    reference = "http://webserver.rilin.state.ri.us/Statutes/title44/44-30/44-30-12.HTM"
    defined_for = "ri_retirement_income_subtraction_eligible"

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        taxable_pension = person("taxable_pension_income", period)
        head_or_spouse = person("is_tax_unit_head_or_spouse", period)
        p = parameters(
            period
        ).gov.states.ri.tax.income.agi.subtractions.taxable_retirement_income
        total_taxable_pension = tax_unit.sum(taxable_pension * head_or_spouse)
        return min_(total_taxable_pension, p.cap)



from policyengine_us.model_api import *


class dc_ptc(Variable):
    value_type = float
    entity = TaxUnit
    label = "DC property tax credit"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://otr.cfo.dc.gov/sites/default/files/dc/sites/otr/publication/attachments/52926_D-40_12.21.21_Final_Rev011122.pdf#page=49"
        "https://otr.cfo.dc.gov/sites/default/files/dc/sites/otr/publication/attachments/2022_D-40_Booklet_Final_blk_01_23_23_Ordc.pdf#page=47"
    )
    defined_for = "takes_up_dc_ptc"

    def formula(tax_unit, period, parameters):
        rent = add(tax_unit, period, ["rent"])
        retax = add(tax_unit, period, ["real_estate_taxes"])
        p_dc = parameters(period).gov.states.dc.tax.income.credits
        ptax = retax + rent * p_dc.ptc.rent_ratio
        elderly_age = p_dc.ptc.min_elderly_age
        head_age = tax_unit("age_head", period)
        spouse_age = tax_unit("age_spouse", period)
        is_elderly = (head_age >= elderly_age) | (spouse_age >= elderly_age)
        us_agi = tax_unit("adjusted_gross_income", period)
        ptax_offset = us_agi * where(
            is_elderly,
            p_dc.ptc.fraction_elderly.calc(us_agi, right=True),
            p_dc.ptc.fraction_nonelderly.calc(us_agi, right=True),
        )
        uncapped_ptc = max_(0, ptax - ptax_offset)
        return min_(p_dc.ptc.max, uncapped_ptc)



from policyengine_us.model_api import *


class nj_income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "New Jersey income tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.NJ

    def formula(tax_unit, period, parameters):
        main_income_tax = tax_unit("nj_main_income_tax", period)
        non_refundable_credits = tax_unit("nj_non_refundable_credits", period)

        return max_(main_income_tax - non_refundable_credits, 0)



from policyengine_us.model_api import *


class dc_taxable_income_joint(Variable):
    value_type = float
    entity = TaxUnit
    label = (
        "DC taxable income (can be negative) when married couple files jointly"
    )
    unit = USD
    definition_period = YEAR
    reference = (
        "https://otr.cfo.dc.gov/sites/default/files/dc/sites/otr/publication/attachments/52926_D-40_12.21.21_Final_Rev011122.pdf#page=36"
        "https://otr.cfo.dc.gov/sites/default/files/dc/sites/otr/publication/attachments/2022_D-40_Booklet_Final_blk_01_23_23_Ordc.pdf#page=34"
    )
    defined_for = StateCode.DC
    adds = ["dc_taxable_income_indiv"]



from policyengine_us.model_api import *


class medicaid_take_up_seed(Variable):
    value_type = float
    entity = Person
    label = "Randomly assigned seed for Medicaid take-up"
    definition_period = YEAR



from policyengine_us.model_api import *


class il_aabd_vehicle_is_essential(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = MONTH
    label = "Whether the household has a vehicle which is considered essential under the Illinois Aid to the Aged, Blind or Disabled (AABD)"
    reference = (
        "https://www.law.cornell.edu/regulations/illinois/Ill-Admin-Code-tit-89-SS-113.141",
    )
    defined_for = StateCode.IL



from policyengine_us.model_api import *


class spm_unit_capped_housing_subsidy_reported(Variable):
    value_type = float
    entity = SPMUnit
    label = "SPM unit capped housing subsidy"
    definition_period = YEAR
    unit = USD
    uprating = "gov.bls.cpi.cpi_u"



from policyengine_us.model_api import *


class wv_sctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "West Virginia senior citizens tax credit"
    defined_for = "wv_sctc_eligible"
    unit = USD
    definition_period = YEAR
    reference = "https://code.wvlegislature.gov/11-21-21/"

    def formula(tax_unit, period, parameters):
        wv_taxable_property_value = tax_unit(
            "wv_taxable_property_value", period
        )
        p = parameters(period).gov.states.wv.tax.income.credits.sctc

        return min_(wv_taxable_property_value, p.max_amount)



from policyengine_us.model_api import *


class ga_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Georgia Child Tax Credit"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.GA
    reference = (
        "https://legiscan.com/GA/text/HB136/id/3204611/Georgia-2025-HB136-Enrolled.pdf#page=2",
    )

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.ga.tax.income.credits.ctc
        person = tax_unit.members
        age = person("age", period)
        ctc_eligible_child = person("ctc_qualifying_child", period)
        ga_child_age_eligible = age < p.age_threshold
        eligible_children = tax_unit.sum(
            ctc_eligible_child & ga_child_age_eligible
        )

        return eligible_children * p.amount



from policyengine_us.model_api import *


class mi_home_heating_credit_eligible_rate(Variable):
    value_type = float
    entity = TaxUnit
    label = "Eligible for the Michigan home heating credit"
    definition_period = YEAR
    defined_for = StateCode.MI

    def formula(tax_unit, period, parameters):
        tax_unit_size = tax_unit("tax_unit_size", period)
        person = tax_unit.members
        dependent_elsewhere = person(
            "claimed_as_dependent_on_another_return", period
        )
        ft_student = person("is_full_time_student", period)

        eligible_person = ~(ft_student & dependent_elsewhere)
        eligible_people = tax_unit.sum(eligible_person)
        # The heating credit is prorated based on the number of eligible people
        # in the tax unit.
        prorate_fraction = np.zeros_like(eligible_people)
        mask = eligible_people > 0
        prorate_fraction[mask] = eligible_people[mask] / tax_unit_size[mask]
        return prorate_fraction



from policyengine_us.model_api import *


class co_income_qualified_senior_housing_credit_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    label = "Eligible for Colorado Income Qualified Senior Housing Income Tax Credit"
    definition_period = YEAR
    reference = (
        "https://tax.colorado.gov/income-qualified-senior-housing-income-tax-credit",
        "https://tax.colorado.gov/sites/tax/files/documents/DR_104_Book_2022.pdf#page=17",
    )
    defined_for = StateCode.CO

    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.co.tax.income.credits.income_qualified_senior_housing

        age_head = tax_unit("age_head", period)
        age_spouse = tax_unit("age_spouse", period)

        head_eligible = age_head >= p.age_limit
        spouse_eligible = age_spouse >= p.age_limit
        age_eligible = head_eligible | spouse_eligible

        agi = tax_unit("adjusted_gross_income", period)
        max_income = p.income_threshold
        agi_eligible = agi <= max_income

        property_tax_exemption_claimed = (
            tax_unit("co_property_tax_exemption", period) > 0
        )

        return age_eligible & agi_eligible & ~property_tax_exemption_claimed



from policyengine_us.model_api import *


class la_general_relief_motor_vehicle_value_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for the Los Angeles County General Relief based on the motor vehicle value requirements"
    definition_period = YEAR
    # Person has to be a resident of LA County
    defined_for = "in_la"
    reference = "https://drive.google.com/file/d/1Oc7UuRFxJj-eDwTeox92PtmRVGnG9RjW/view?usp=sharing"

    def formula(spm_unit, period, parameters):
        household = spm_unit.household
        lives_in_vehicle = household("lives_in_vehicle", period)
        vehicle_value = household("household_vehicles_value", period)
        p = parameters(
            period
        ).gov.local.ca.la.general_relief.eligibility.limit.motor_vehicle
        vehicle_value_limit = where(
            lives_in_vehicle, p.value.homeless, p.value.resident
        )
        vehicle_value_eligible = vehicle_value <= vehicle_value_limit
        vehicles_owned = household("household_vehicles_owned", period)
        return vehicle_value_eligible & (vehicles_owned <= p.cap)



from policyengine_us.model_api import *


class DCLIHEAPHeatingType(Enum):
    ELECTRICITY = "Electricity"
    GAS = "Gas"
    HEAT_IN_RENT = "Heat in Rent"  # Electricity or gas included in rent
    OIL = "Oil"


class dc_liheap_heating_type(Variable):
    value_type = Enum
    entity = SPMUnit
    possible_values = DCLIHEAPHeatingType
    default_value = DCLIHEAPHeatingType.ELECTRICITY
    definition_period = YEAR
    label = "Household heating types for DC LIHEAP"



from policyengine_us.model_api import *


class de_non_refundable_eitc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Delaware non-refundable EITC"
    unit = USD
    documentation = "Non-refundable EITC credit reducing DE State income tax."
    definition_period = YEAR
    reference = "https://revenuefiles.delaware.gov/2022/PIT-RES_TY22_2022-02_Instructions.pdf"
    defined_for = StateCode.DE

    def formula(tax_unit, period, parameters):
        # Either claims refundable or non-refundable, but not both.
        claims = ~tax_unit("de_claims_refundable_eitc", period)
        amount_if_claimed = tax_unit(
            "de_non_refundable_eitc_if_claimed", period
        )
        return claims * amount_if_claimed



from policyengine_us.model_api import *


class mt_disability_income_exclusion(Variable):
    value_type = float
    entity = TaxUnit
    label = "Montana disability income exclusion"
    defined_for = StateCode.MT
    unit = USD
    definition_period = YEAR
    reference = "https://rules.mt.gov/gateway/RuleNo.asp?RN=42%2E15%2E217"
    adds = ["mt_disability_income_exclusion_person"]



from policyengine_us.model_api import *


class al_casualty_loss_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Alabama casualty loss deduction"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.AL

    def formula(tax_unit, period, parameters):
        loss = add(tax_unit, period, ["casualty_loss"])
        deduction = parameters(period).gov.irs.deductions.itemized.casualty
        al_agi = tax_unit("al_agi", period)
        return max_(0, loss - al_agi * deduction.floor)



from policyengine_us.model_api import *


class capped_count_cdcc_eligible(Variable):
    value_type = int
    entity = TaxUnit
    label = "Capped child/dependent care eligiable count"
    definition_period = YEAR
    reference = (
        "https://www.law.cornell.edu/uscode/text/26/21#c",
        "https://www.law.cornell.edu/uscode/text/26/21#d_1",
    )

    def formula(tax_unit, period, parameters):
        cdcc = parameters(period).gov.irs.credits.cdcc
        eligible_people = tax_unit("count_cdcc_eligible", period)
        return min_(cdcc.eligibility.max, eligible_people)



from policyengine_us.model_api import *


class mt_interest_exemption_person(Variable):
    value_type = float
    entity = Person
    label = "Montana interest exemption for each person"
    unit = USD
    definition_period = YEAR
    reference = "https://mtrevenue.gov/wp-content/uploads/dlm_uploads/2022/12/Form-2-2022-Instructions.pdf#page=25"
    defined_for = "mt_interest_exemption_eligible_person"

    def formula(person, period, parameters):
        # Allocate the interest exemption to head/spouse based on share of interest income.
        interest_income = person("taxable_interest_income", period)
        total_interest_income = person.tax_unit.sum(interest_income)
        total_deduction = person.tax_unit("mt_interest_exemption", period)
        deduction_rate = np.zeros_like(total_interest_income)
        mask = total_interest_income != 0
        deduction_rate[mask] = (
            interest_income[mask] / total_interest_income[mask]
        )
        return total_deduction * deduction_rate



from policyengine_us.model_api import *


class la_general_relief_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for the Los Angeles County General Relief"
    definition_period = MONTH
    # Person has to be a resident of LA County
    defined_for = "in_la"
    reference = "https://drive.google.com/file/d/1Oc7UuRFxJj-eDwTeox92PtmRVGnG9RjW/view?usp=sharing"

    def formula(spm_unit, period, parameters):
        age_eligible = spm_unit("la_general_relief_age_eligible", period)
        cash_eligible = spm_unit(
            "la_general_relief_cash_asset_eligible", period
        )
        home_value_eligible = spm_unit(
            "la_general_relief_home_value_eligible", period
        )
        motor_vehicle_value_eligible = spm_unit(
            "la_general_relief_motor_vehicle_value_eligible", period
        )
        net_income_eligible = spm_unit(
            "la_general_relief_net_income_eligible", period
        )
        personal_property_eligible = spm_unit(
            "la_general_relief_personal_property_eligible", period
        )
        disability_eligible = spm_unit(
            "la_general_relief_disability_eligible", period
        )
        immigration_status_eligible = spm_unit(
            "la_general_relief_immigration_status_eligible", period
        )
        return (
            age_eligible
            & cash_eligible
            & home_value_eligible
            & motor_vehicle_value_eligible
            & net_income_eligible
            & personal_property_eligible
            & immigration_status_eligible
        )



from policyengine_us.model_api import *


class capped_advanced_main_air_circulating_fan_credit(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Capped advanced main air circulating fan credit"
    documentation = "Capped advanced main air circulating fan credit"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/25C#b_3_A"

    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.irs.credits.energy_efficient_home_improvement
        expenditure = tax_unit(
            "advanced_main_air_circulating_fan_expenditures", period
        )
        rate = p.rates.property
        uncapped = expenditure * rate
        # Cap at either the fan cap (pre-IRA) or total property cap (post-IRA).
        cap = min_(
            p.cap.annual.advanced_main_air_circulating_fan,
            p.cap.annual.energy_efficient_building_property,
        )
        return min_(uncapped, cap)



## line_45 tax form
from policyengine_us.model_api import *


class mi_alternate_home_heating_credit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Michigan alternate home heating credit amount"
    defined_for = "mi_alternate_home_heating_credit_eligible"
    unit = USD
    definition_period = YEAR

    reference = (
        "https://www.michigan.gov/-/media/Project/Websites/taxes/2022RM/IIT/MI-1040CR7.pdf?rev=84f72df3f8664b96903aa6b655dc34d2"
        "http://www.legislature.mi.gov/(S(keapvg1h2vndkn25rtmpyyse))/mileg.aspx?page=getObject&objectName=mcl-206-527a"
    )

    # Calculated from https://drive.google.com/file/d/1-bCwQn3nV9W-bIPlrVViTHes0ovWgLBr/view?usp=sharing
    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.mi.tax.income.credits.home_heating.alternate

        household_resources = tax_unit("mi_household_resources", period)
        heating_expenses = tax_unit("heating_expenses", period)
        # Line 42
        capped_heating_expenses = min_(heating_expenses, p.heating_costs.cap)
        # Line 43
        reduced_household_resources = (
            household_resources * p.household_resources.rate
        )
        # Line 44
        reduced_capped_heating_costs = max_(
            capped_heating_expenses - reduced_household_resources, 0
        )
        # Line 45
        return p.heating_costs.rate * reduced_capped_heating_costs



from policyengine_us.model_api import *


class tx_tanf_income_limit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Texas TANF income limit"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.hhs.texas.gov/services/financial/cash/tanf-cash-help"
    )
    defined_for = StateCode.TX

    def formula(spm_unit, period, parameters):
        size = spm_unit("spm_unit_size", period)
        p = parameters(period).gov.states.tx.tanf
        return p.monthly_income_limit.calc(size) * MONTHS_IN_YEAR



from policyengine_us.model_api import *


class ca_state_supplement_medical_care_facility_amount(Variable):
    value_type = float
    entity = SPMUnit
    label = "California SSI state supplement medical care facility amount"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.CA
    reference = "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?lawCode=WIC&sectionNum=12200"

    def formula(spm_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.ca.cdss.state_supplement.payment_standard
        # Medical care facility amount
        person = spm_unit.members
        eligible = person("ca_state_supplement_eligible_person", period)
        in_medical_care_facility = person(
            "ca_in_medical_care_facility", period
        )
        medical_care_facility_count = spm_unit.sum(
            in_medical_care_facility * eligible
        )
        return medical_care_facility_count * p.allowance.medical_care_facility



from policyengine_us.model_api import *


class mo_net_state_income_taxes(Variable):
    value_type = float
    entity = TaxUnit
    label = "Missouri net state income taxes"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://dor.mo.gov/forms/MO-A_2021.pdf#page=2",
        "https://revisor.mo.gov/main/OneSection.aspx?section=143.141&bid=7212",
    )
    defined_for = StateCode.MO

    def formula(tax_unit, period, parameters):
        # follows Form MO-A Part II and Part II worksheet logic:

        filing_status = tax_unit("filing_status", period)
        p = parameters(period).gov.irs.deductions.itemized
        salt_cap = p.salt_and_real_estate.cap[filing_status]

        uncapped_itax = max_(
            0, add(tax_unit, period, ["state_withheld_income_tax"])
        )
        uncapped_ptax = add(tax_unit, period, ["real_estate_taxes"])
        uncapped_salt = uncapped_itax + uncapped_ptax

        ratio = np.zeros_like(uncapped_salt)
        mask = uncapped_salt != 0
        ratio[mask] = uncapped_itax[mask] / uncapped_salt[mask]

        return where(uncapped_salt > salt_cap, salt_cap * ratio, uncapped_itax)



from policyengine_us.model_api import *


class salt_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "SALT deduction"
    unit = USD
    documentation = "State and local taxes plus real estate tax deduction from taxable income."
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/164"

    def formula(tax_unit, period, parameters):
        reported_salt = tax_unit("reported_salt", period)
        cap = tax_unit("salt_cap", period)
        return min_(cap, reported_salt)



from policyengine_us.model_api import *


class me_property_tax_fairness_credit_veterans_cap(Variable):
    value_type = float
    entity = TaxUnit
    unit = USD
    label = "Veterans cap for Maine property tax fairness credit"
    definition_period = YEAR
    defined_for = StateCode.ME

    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.me.tax.income.credits.fairness.property_tax
        base_cap = tax_unit("me_property_tax_fairness_credit_base_cap", period)
        return base_cap * p.veterans_matched



from policyengine_us.model_api import *


class ar_post_secondary_education_tuition_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Arkansas post-secondary education tuition deduction"
    unit = USD
    definition_period = YEAR
    reference = "https://www.dfa.arkansas.gov/images/uploads/incomeTaxOffice/AR1075_2022.pdf#page=1"
    defined_for = StateCode.AR

    adds = ["ar_post_secondary_education_tuition_deduction_person"]



from policyengine_us.model_api import *


class co_modified_agi(Variable):
    value_type = float
    entity = TaxUnit
    label = "Colorado modified adjusted gross income for the sales tax refund"
    unit = USD
    definition_period = YEAR
    reference = "https://tax.colorado.gov/sites/tax/files/documents/DR_0104_Book_2022.pdf#page=23"
    defined_for = StateCode.CO

    adds = "gov.states.co.tax.income.credits.sales_tax_refund.magi_sources"



from policyengine_us.model_api import *


def create_reported_state_income_tax() -> Reform:
    class household_tax_before_refundable_credits(Variable):
        value_type = float
        entity = Household
        label = "total tax before refundable credits"
        documentation = "Total tax liability before refundable credits."
        unit = USD
        definition_period = YEAR

        def formula(household, period, parameters):
            p = parameters(period)
            added_components = (
                p.gov.household.household_tax_before_refundable_credits
            )
            flat_tax = p.gov.contrib.ubi_center.flat_tax
            if p.simulation.reported_state_income_tax:
                added_components = [
                    "employee_payroll_tax",
                    "self_employment_tax",
                    "income_tax_before_refundable_credits",  # Federal.
                    "flat_tax",
                    # State tax reported in ASEC includes refundable credits.
                    "spm_unit_state_tax_reported",
                ]
            if flat_tax.abolish_payroll_tax:
                added_components = [
                    c for c in added_components if c != "employee_payroll_tax"
                ]
            if flat_tax.abolish_federal_income_tax:
                added_components = [
                    c
                    for c in added_components
                    if c != "income_tax_before_refundable_credits"
                ]
            return add(household, period, added_components)

    class household_refundable_tax_credits(Variable):
        value_type = float
        entity = Household
        label = "refundable tax credits"
        definition_period = YEAR
        unit = USD

        adds = ["income_tax_refundable_credits"]

    class reform(Reform):
        def apply(self):
            self.update_variable(household_tax_before_refundable_credits)
            self.update_variable(household_refundable_tax_credits)

    return reform


def create_reported_state_income_tax_reform(
    parameters, period, bypass: bool = False
):
    if bypass:
        return create_reported_state_income_tax()

    p = parameters(period).simulation

    if p.reported_state_income_tax:
        return create_reported_state_income_tax()
    else:
        return None


reported_state_income_tax = create_reported_state_income_tax_reform(
    None, None, bypass=True
)



from policyengine_us.model_api import *


class dc_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "DC Temporary Assistance for Needy Families (TANF) countable earned income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.DC
    reference = (
        "https://code.dccouncil.gov/us/dc/council/code/sections/4-205.11"
    )

    adds = ["dc_tanf_earned_income_after_disregard_person"]



from policyengine_us.model_api import *


class ct_income_tax_after_amt(Variable):
    value_type = float
    entity = TaxUnit
    label = "Connecticut income tax after the addition of the alternative minimum tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.CT
    reference = (
        "https://portal.ct.gov/-/media/DRS/Forms/2022/Income/2022-CT-1040-Instructions_1222.pdf#page=2"  # line 10
        "https://portal.ct.gov/-/media/DRS/Forms/2022/Income/CT-1040_1222.pdf#page=1"  # line 10
    )

    adds = ["ct_amt", "ct_income_tax_after_personal_credits"]



from policyengine_us.model_api import *


class tip_income(Variable):
    value_type = float
    entity = Person
    label = "Tip income"
    unit = USD
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/cfr/text/26/31.3402(k)-1"

    # This variable only exists for the purpose of the tax_exempt_reform



from policyengine_us.model_api import *


class nyc_school_tax_credit_rate_reduction_amount(Variable):
    value_type = float
    entity = TaxUnit
    label = "NYC School Tax Credit Rate Reduction Amount"
    unit = USD
    definition_period = YEAR
    defined_for = "nyc_school_tax_credit_rate_reduction_amount_eligible"

    def formula(tax_unit, period, parameters):
        # First get their NYC taxable income.
        nyc_taxable_income = tax_unit("nyc_taxable_income", period)

        # Then get their filing status.
        filing_status = tax_unit("filing_status", period)

        # Then get the School Tax Credit rate reduction amount part of the parameter tree.
        p = parameters(
            period
        ).gov.local.ny.nyc.tax.income.credits.school.rate_reduction

        # Calculate amount if eligible, which varies only with filing status.
        filing_statuses = filing_status.possible_values
        return select(
            [
                filing_status == filing_statuses.SINGLE,
                filing_status == filing_statuses.JOINT,
                filing_status == filing_statuses.SEPARATE,
                filing_status == filing_statuses.HEAD_OF_HOUSEHOLD,
                filing_status == filing_statuses.SURVIVING_SPOUSE,
            ],
            [
                p.amount.single.calc(nyc_taxable_income),
                p.amount.joint.calc(nyc_taxable_income),
                p.amount.separate.calc(nyc_taxable_income),
                p.amount.head_of_household.calc(nyc_taxable_income),
                p.amount.surviving_spouse.calc(nyc_taxable_income),
            ],
        )



from policyengine_us.model_api import *


class CaCalworksChildCareTimeCategory(Enum):
    HOURLY = "Hourly"
    DAILY = "Daily"
    WEEKLY = "Weekly"
    MONTHLY = "Monthly"


class ca_calworks_child_care_time_category(Variable):
    value_type = Enum
    possible_values = CaCalworksChildCareTimeCategory
    default_value = CaCalworksChildCareTimeCategory.WEEKLY
    entity = Person
    label = "California CalWORKs Child Care time category"
    definition_period = MONTH
    defined_for = StateCode.CA
    reference = "http://epolicy.dpss.lacounty.gov/epolicy/epolicy/server/general/projects_responsive/ePolicyMaster/index.htm?&area=general&type=responsivehelp&ctxid=&project=ePolicyMaster#t=mergedProjects%2FChild%20Care%2FChild_Care%2F1210_8_Regional_Market_Rate_Ceilings%2F1210_8_Regional_Market_Rate_Ceilings.htm%23Contactbc-13&rhtocid=_3_3_8_12"
    # Depends on hours of care received per day or week.
    # We do not currently model this.



from policyengine_us.model_api import *


class oh_insured_unreimbursed_medical_care_expense_amount(Variable):
    value_type = float
    entity = Person
    label = "Ohio insured unreimbursed medical and health care expense amount"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://tax.ohio.gov/static/forms/ohio_individual/individual/2022/it1040-sd100-instruction-booklet.pdf#page=18",  # Line 36
        "https://tax.ohio.gov/static/forms/ohio_individual/individual/2022/it1040-sd100-instruction-booklet.pdf#page=27",
        "https://codes.ohio.gov/ohio-revised-code/section-5747.01",  # R.C. 5747.01(10)
    )
    defined_for = StateCode.OH

    def formula(person, period, parameters):
        employer_premium_contribution = person(
            "employer_contribution_to_health_insurance_premiums_category",
            period,
        )
        status = employer_premium_contribution.possible_values
        medicare_eligible = person("is_medicare_eligible", period)
        # Line 3
        eligible_premiums = (
            person("health_insurance_premiums", period) * medicare_eligible
        ) * (employer_premium_contribution == status.NONE)
        # Premiums only count if the employer paid none.
        # Line 4
        medical_out_of_pocket_expenses = person(
            "medical_out_of_pocket_expenses", period
        )
        return eligible_premiums + medical_out_of_pocket_expenses



from policyengine_us.model_api import *


class nj_taxable_income(Variable):
    value_type = float
    entity = TaxUnit
    label = "New Jersey taxable income"
    unit = USD
    documentation = "NJ AGI less taxable income deductions (Line 42)"
    definition_period = YEAR
    reference = "https://law.justia.com/codes/new-jersey/2022/title-54a/section-54a-3-1/"
    defined_for = StateCode.NJ

    def formula(tax_unit, period, parameters):
        # Get taxable income before property tax deduction or credit.
        taxable_income_before_deduction = tax_unit(
            "nj_taxable_income_before_property_tax_deduction", period
        )

        # Get the property tax deduction.
        property_tax_deduction = tax_unit("nj_property_tax_deduction", period)

        return max_(
            0, taxable_income_before_deduction - property_tax_deduction
        )



from policyengine_us.model_api import *


class taxsim_taxsimid(Variable):
    value_type = int
    entity = TaxUnit
    label = "Tax unit ID"
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        return tax_unit("tax_unit_id", period)



from policyengine_us.model_api import *


class vt_charitable_contribution_credit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Vermont charitable contribution credit"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://law.justia.com/codes/vermont/2022/title-32/chapter-151/section-5822/"
        "https://tax.vermont.gov/sites/tax/files/documents/IN-111-2022.pdf#page=1"
    )
    defined_for = StateCode.VT

    def formula(tax_unit, period, parameters):
        charitable_contributions = add(
            tax_unit,
            period,
            ["charitable_cash_donations", "charitable_non_cash_donations"],
        )
        p = parameters(period).gov.states.vt.tax.income.credits
        return p.charitable.calc(charitable_contributions)



from policyengine_us.model_api import *


class current_pregnancies(Variable):
    value_type = int
    entity = Person
    definition_period = YEAR
    label = "The number of children a pregnant person is expecting"



from policyengine_us.model_api import *


class ma_liheap_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Massachusetts LIHEAP due to income"
    definition_period = YEAR
    defined_for = StateCode.MA
    reference = "https://www.mass.gov/info-details/learn-about-home-energy-assistance-heap"

    def formula(spm_unit, period, parameters):
        income = spm_unit("ma_liheap_income", period)
        income_threshold = spm_unit(
            "ma_liheap_state_median_income_threshold", period
        )
        return income <= income_threshold



from policyengine_us.model_api import *


class claimed_ma_covid_19_essential_employee_premium_pay_program_2020(
    Variable
):
    value_type = bool
    entity = Person
    label = (
        "Claimed MA COVID 19 Essential Employee Premium Pay Program for 2020"
    )
    definition_period = YEAR
    reference = "https://www.mass.gov/info-details/covid-19-essential-employee-premium-pay-program"
    defined_for = StateCode.MA



from policyengine_us.model_api import *


class used_clean_vehicle_credit_eligible(Variable):
    value_type = bool
    entity = TaxUnit
    definition_period = YEAR
    label = "Eligible for used clean vehicle credit"
    documentation = "Eligible for nonrefundable credit for the purchase of a previously-owned clean vehicle"
    unit = USD
    reference = "https://www.democrats.senate.gov/imo/media/doc/inflation_reduction_act_of_2022.pdf#page=370"
    defined_for = "purchased_qualifying_used_clean_vehicle"

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.irs.credits.clean_vehicle.used
        # Income eligibility based on lesser of MAGI in current and prior year.
        # Assume AGI in current year for now.
        agi = tax_unit("adjusted_gross_income", period)
        filing_status = tax_unit("filing_status", period)
        income_limit = p.eligibility.income_limit[filing_status]
        income_eligible = agi <= income_limit
        # Purchase price limit.
        sale_price = tax_unit("used_clean_vehicle_sale_price", period)
        price_eligible = sale_price <= p.eligibility.sale_price_limit
        return income_eligible & price_eligible



from policyengine_us.model_api import *


class non_public_school_tuition(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    label = "Nonchartered, Nonpublic School Tuition"



from policyengine_us.model_api import *


class slcsp_age_curve_applies(Variable):
    value_type = bool
    entity = TaxUnit
    label = "ACA age curve applies, rather than family tier"
    definition_period = MONTH

    def formula(tax_unit, period, parameters):
        family_tier_applies = tax_unit("slcsp_family_tier_applies", period)
        return ~family_tier_applies



from policyengine_us.model_api import *


class is_tax_unit_spouse(Variable):
    value_type = bool
    entity = Person
    label = "Spouse of tax unit"
    definition_period = YEAR

    def formula(person, period, parameters):
        # Only non-head adults can be spouses.
        is_separated = person.tax_unit.any(person("is_separated", period))
        adult = ~person("is_child", period)
        head = person("is_tax_unit_head", period)
        eligible = adult & ~head & ~is_separated
        tax_unit = person.tax_unit
        age = person("age", period)
        return person.get_rank(tax_unit, -age, eligible) == 0



from policyengine_us.model_api import *


class co_chp_out_of_pocket_maximum(Variable):
    value_type = float
    entity = Person
    label = "Colorado Child Health Plan Plus out of pocket maximum"
    definition_period = YEAR

    def formula(person, period, parameters):
        income = person.tax_unit("medicaid_magi", period)
        percent = parameters(period).gov.states.co.hcpf.chp.out_of_pocket
        return income * percent



from policyengine_us.model_api import *


class is_acp_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Affordable Connectivity Program"
    documentation = "Eligible for Affordable Connectivity Program"
    definition_period = YEAR
    # 47 U.S.C §1752(a)(6).
    reference = "https://uscode.house.gov/view.xhtml?req=granuleid:USC-prelim-title47-section1752&edition=prelim"

    def formula(spm_unit, period, parameters):
        fcc = parameters(period).gov.fcc
        categorically_eligible = np.any(
            [
                add(spm_unit, period, [program])
                for program in fcc.acp.categorical_eligibility
            ],
            axis=0,
        )
        # ACP categorical eligibility points includes Lifeline categorical eligibility.
        household = spm_unit.household
        is_on_tribal_land = household("is_on_tribal_land", period)
        non_tribal_lifeline_programs = add(
            spm_unit, period, fcc.lifeline.categorical_eligibility
        )
        tribal_lifeline_programs = add(
            spm_unit, period, fcc.lifeline.tribal_categorical_eligibility
        )
        lifeline_categorically_eligible = np.where(
            is_on_tribal_land,
            np.any(tribal_lifeline_programs),
            np.any(non_tribal_lifeline_programs),
        )

        fpg_eligible = spm_unit("fcc_fpg_ratio", period) <= fcc.acp.fpg_limit
        # Cannot be simultaneously enrolled in Emergency Broadband Benefit.
        ebb_enrolled = spm_unit("ebb", period) > 0
        return (
            categorically_eligible
            | fpg_eligible
            | lifeline_categorically_eligible
        ) & ~ebb_enrolled



from policyengine_us.model_api import *


class md_personal_exemption(Variable):
    value_type = float
    entity = TaxUnit
    label = "MD value per personal exemption"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MD
    reference = "https://casetext.com/statute/code-of-maryland/article-tax-general/title-10-income-tax/subtitle-2-maryland-taxable-income-calculations-for-individual/part-iii-exemptions/section-10-211-individuals-other-than-fiduciaries?searchWithin=true&listingIndexId=code-of-maryland.article-tax-general&q=blind&type=statute&sort=relevance&p=1"

    def formula(tax_unit, period, parameters):
        # Get filing status and AGI.
        filing_status = tax_unit("filing_status", period)
        filing_statuses = filing_status.possible_values
        agi = tax_unit("adjusted_gross_income", period)
        # Calculate for each filing status depending on AGI.
        p = parameters(period).gov.states.md.tax.income.exemptions.personal
        return select(
            [
                filing_status == filing_statuses.SINGLE,
                filing_status == filing_statuses.SEPARATE,
                filing_status == filing_statuses.JOINT,
                filing_status == filing_statuses.HEAD_OF_HOUSEHOLD,
                filing_status == filing_statuses.SURVIVING_SPOUSE,
            ],
            [
                p.single.calc(agi, right=True),
                p.separate.calc(agi, right=True),
                p.joint.calc(agi, right=True),
                p.head.calc(agi, right=True),
                p.surviving_spouse.calc(agi, right=True),
            ],
        )



from policyengine_us.model_api import *


class va_spouse_tax_adjustment(Variable):
    value_type = float
    entity = TaxUnit
    label = "Virginia Spouse Tax Adjustment"
    defined_for = "va_spouse_tax_adjustment_eligible"
    unit = USD
    definition_period = YEAR
    reference = "https://www.tax.virginia.gov/sites/default/files/vatax-pdf/2022-760-instructions.pdf#page=19"

    def formula(tax_unit, period, parameters):
        person = tax_unit.members
        head_or_spouse = person("is_tax_unit_head_or_spouse", period)
        # Line 4, enter the taxable income
        taxable_income = tax_unit("va_taxable_income", period)
        # Line 5, enter the AGI less exemptions:
        agi_less_exemptions = person("va_agi_less_exemptions_person", period)
        capped_agi_less_exemptions = max_(agi_less_exemptions, 0)
        # If the tax filer is the head or the spouse of the tax unit, take his/hers corresponding value
        # Set the value for dependent amount to infinity as we will need the smaller amount for
        # further calculations
        smaller_agi_less_exemptions = tax_unit.min(
            where(head_or_spouse, capped_agi_less_exemptions, np.inf)
        )
        # Line 6, subtract
        reduced_taxable_income = max_(
            taxable_income - smaller_agi_less_exemptions, 0
        )
        # Line 7, divide taxable income in half
        p = parameters(
            period
        ).gov.states.va.tax.income.exemptions.spouse_tax_adjustment
        half_of_taxable_income = taxable_income / p.divisor
        # Line 8, take the smaller of the tax calculated on line 5 or line 7
        p1 = parameters(period).gov.states.va.tax.income
        smaller_agi_or_taxable_income = min_(
            p1.rates.calc(smaller_agi_less_exemptions),
            p1.rates.calc(half_of_taxable_income),
        )
        # Line 9, enter the larger of the tax calculated on line 6 or line 7
        larger_reduced_taxable_income_or_halved_taxable_income = max_(
            p1.rates.calc(reduced_taxable_income),
            p1.rates.calc(half_of_taxable_income),
        )
        # Line 10, add line 8 and line 9
        addition_of_tax = (
            smaller_agi_or_taxable_income
            + larger_reduced_taxable_income_or_halved_taxable_income
        )
        # Line 11, enter income tax before credits
        income_tax_before_credits = tax_unit(
            "va_income_tax_before_non_refundable_credits", period
        )
        # Line 12, subtract line 10 from line 11
        reduced_tax = max_(income_tax_before_credits - addition_of_tax, 0)
        # The value cannot exceed a certain threshold
        return min_(reduced_tax, p.cap)



from policyengine_us.model_api import *


class or_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "Oregon Child Tax Credit"
    unit = USD
    definition_period = YEAR
    reference = "https://olis.oregonlegislature.gov/liz/2023R1/Downloads/MeasureDocument/HB3235/Enrolled"
    defined_for = StateCode.OR

    def formula(tax_unit, period, parameters):
        # Get age and dependent status of all people in the tax unit.
        person = tax_unit.members
        age = person("age", period)
        dependent = person("is_tax_unit_dependent", period)
        # Get the number of qualifying dependents in the tax unit.
        p = parameters(period).gov.states["or"].tax.income.credits.ctc
        age_eligible = age < p.ineligible_age
        eligible = age_eligible & dependent
        count_eligible = tax_unit.sum(eligible)
        # Cap the number of qualifying dependents.
        capped_count_eligible = min_(count_eligible, p.child_limit)
        # Get maximum credit amount.
        max_credit = p.amount * capped_count_eligible
        # Get Oregon adjusted gross income.
        or_agi = tax_unit("or_agi", period)
        # Reduce credit amount over the phaseout range.
        excess_agi = max_(or_agi - p.reduction.start, 0)
        percent_reduction = min_(excess_agi / p.reduction.width, 1)
        # Return reduced amount.
        return max_credit * (1 - percent_reduction)



from policyengine_us.model_api import *


class nc_scca_entry_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "North Carolina entry eligibility for Subsidized Child Care Assistance Program"
    reference = "https://ncchildcare.ncdhhs.gov/Portals/0/documents/pdf/A/ACF-118_CCDF_FFY_2022-2024_For_North_Carolina_Amendment_1.pdf?ver=C9YfIUPAFekeBA3I1mN8aA%3d%3d#page=83"
    definition_period = MONTH
    defined_for = StateCode.NC

    def formula(spm_unit, period, parameters):
        income_eligible = spm_unit("nc_scca_entry_income_eligible", period)
        has_eligible_children = spm_unit("nc_scca_has_eligible_child", period)

        return income_eligible & has_eligible_children



from policyengine_us.model_api import *


class is_pregnant(Variable):
    value_type = bool
    entity = Person
    label = "Is pregnant"
    definition_period = YEAR
    adds = ["current_pregnancies"]



from policyengine_us.model_api import *


class mt_child_dependent_care_expense_deduction(Variable):
    value_type = float
    entity = Person
    label = "Montana child dependent care expense deduction"
    unit = USD
    definition_period = YEAR
    reference = "https://casetext.com/statute/montana-code/title-15-taxation/chapter-30-individual-income-tax/part-21-rate-and-general-provisions/section-15-30-2131-repealed-effective-112024-temporary-deductions-allowed-in-computing-net-income"
    defined_for = StateCode.MT

    def formula(person, period, parameters):
        tax_unit = person.tax_unit
        # Line 1
        eligible_children = tax_unit(
            "mt_child_dependent_care_expense_deduction_eligible_children",
            period,
        )
        p = parameters(
            period
        ).gov.states.mt.tax.income.deductions.child_dependent_care_expense
        cap = p.cap.calc(eligible_children)
        care_expenses = tax_unit("tax_unit_childcare_expenses", period)
        # Line 2
        capped_expenses = min_(care_expenses, cap)
        # Line 3
        agi = person("mt_agi", period)
        # Line 6
        reduction = p.phase_out.calc(agi)
        # The deduction has to be allocated equally between spouses
        head_or_spouse = person("is_tax_unit_head_or_spouse", period)
        return head_or_spouse * (max_(0, capped_expenses - reduction) * 0.5)



from policyengine_us.model_api import *


class refundable_payroll_tax_credit(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Refundable Payroll Tax Credit"
    unit = USD



from policyengine_us.model_api import *


class spm_unit_spm_expenses(Variable):
    value_type = float
    entity = SPMUnit
    label = "SPM unit's SPM expenses (other than taxes)"
    definition_period = YEAR
    unit = USD

    adds = [
        "child_support_expense",
        "medical_out_of_pocket_expenses",
        "spm_unit_capped_work_childcare_expenses",
    ]



from policyengine_us.model_api import *


class ny_itemized_deductions_reduction_based_on_charitable_deduction_applies(
    Variable
):
    value_type = bool
    entity = TaxUnit
    label = (
        "New York itemized deductions reduction based on charitable deduction"
    )
    definition_period = YEAR
    reference = "https://www.nysenate.gov/legislation/laws/TAX/615"  # (g)
    defined_for = StateCode.NY

    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.ny.tax.income.deductions.itemized.reduction
        agi = tax_unit("ny_agi", period)
        high_income_threshold = p.charitable_deduction_rate.thresholds[1]
        return agi > high_income_threshold



from policyengine_us.model_api import *


class ma_tafdc_partially_disregarded_earned_income(Variable):
    value_type = float
    unit = USD
    entity = Person
    label = "Massachusetts Temporary Assistance for Families with Dependent Children (TAFDC) partially disregarded earned income"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/massachusetts/106-CMR-704-281"
    )
    defined_for = StateCode.MA

    def formula(person, period, parameters):
        work_related_deduction = person(
            "ma_tafdc_work_related_expense_deduction", period
        )
        p = parameters(
            period
        ).gov.states.ma.dta.tcap.tafdc.earned_income_disregard

        gross_earned_income = person("ma_tcap_gross_earned_income", period)
        earned_income_after_work_related_deduction = max_(
            0, gross_earned_income - work_related_deduction
        )
        filing_status = person.tax_unit("filing_status", period)
        joint = filing_status == filing_status.possible_values.JOINT
        dependent_care_deduction = person.spm_unit(
            "ma_tafdc_dependent_care_deduction", period
        )
        # Attributing the dependent care deduction equally to each spouse if filing jointly
        applicable_dependent_care_deduction = where(
            joint, dependent_care_deduction / 2, dependent_care_deduction
        )
        return max_(
            0,
            earned_income_after_work_related_deduction * p.percentage
            - applicable_dependent_care_deduction,
        )



from policyengine_us.model_api import *

# calculated by subtracting the resident credit, which is currently not modeled


class pa_eligibility_income(Variable):
    value_type = float
    entity = TaxUnit
    label = "PA eligibility income"
    unit = USD
    definition_period = YEAR
    reference = "https://www.revenue.pa.gov/FormsandPublications/FormsforIndividuals/PIT/Documents/2021/2021_pa-40sp.pdf"
    defined_for = StateCode.PA

    adds = "gov.states.pa.tax.income.forgiveness.eligibility_income_sources"
